"""Tests for views of contributions (media files)."""

import json
import os
import glob

from os.path import dirname, normpath, abspath, join

from PIL import Image
from io import BytesIO

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.contrib.auth.models import AnonymousUser
from django.core.files import File
from django.core.files.base import ContentFile
from django.core.exceptions import PermissionDenied
from django.conf import settings

from nose.tools import raises
from rest_framework.test import APIRequestFactory, force_authenticate
from rest_framework.renderers import JSONRenderer

from geokey.core.exceptions import MalformedRequestData, FileTypeError
from geokey.core.tests.helpers.image_helpers import get_image
from geokey.projects.tests.model_factories import UserFactory, ProjectFactory
from geokey.contributions.models import MediaFile
from geokey.users.models import User

from geokey.contributions.views.media import (
    MediaAbstractAPIView,
    MediaAPIView,
    SingleMediaAPIView
)

from ..model_factories import ObservationFactory
from .model_factories import ImageFileFactory


class MediaFileAbstractListAPIViewTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserFactory.create()
        self.creator = UserFactory.create()
        self.project = ProjectFactory(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )

        self.contribution = ObservationFactory.create(
            **{'project': self.project}
        )

    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/**/*'
        ))
        for f in files:
            os.remove(f)

    def render(self, response):
        response.accepted_renderer = JSONRenderer()
        response.accepted_media_type = 'application/json'
        response.renderer_context = {'blah': 'blubb'}
        return response.render()

    def test_get_list_and_respond(self):
        ImageFileFactory.create_batch(5, **{'contribution': self.contribution})

        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        request = self.factory.get(url)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        response = self.render(
            view.get_list_and_respond(request, self.contribution)
        )
        self.assertEqual(len(json.loads(response.content)), 5)

    def test_create_image_and_respond(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        data = {
            'name': 'A test image',
            'description': 'Test image description',
            'file': get_image()
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        response = self.render(
            view.create_and_respond(request, self.contribution)
        )

        response_json = json.loads(response.content)
        self.assertEqual(
            response_json.get('name'),
            data.get('name')
        )
        self.assertEqual(
            response_json.get('description'),
            data.get('description')
        )
        self.assertEqual(
            response_json.get('creator').get('display_name'),
            request.user.display_name
        )
        self.assertEqual(
            response_json.get('file_type'),
            'ImageFile'
        )
        self.assertIsNotNone(response_json.get('url'))

    @raises(MalformedRequestData)
    def test_create_image_and_respond_without_file(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        data = {
            'name': 'A test image',
            'description': 'Test image description'
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        view.create_and_respond(request, self.contribution)

    @raises(MalformedRequestData)
    def test_create_image_and_respond_without_name(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        data = {
            'description': 'Test image description',
            'file': get_image()
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        view.create_and_respond(request, self.contribution)

    def test_create_document_and_respond(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        document = File(open(
            normpath(join(
                dirname(abspath(__file__)),
                'files/document_1.pdf'
            )),
            'rb'
        ))

        data = {
            'name': 'A test document',
            'description': 'Test document description',
            'file': document
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        response = self.render(
            view.create_and_respond(request, self.contribution)
        )

        response_json = json.loads(response.content)
        self.assertEqual(
            response_json.get('name'),
            data.get('name')
        )
        self.assertEqual(
            response_json.get('description'),
            data.get('description')
        )
        self.assertEqual(
            response_json.get('creator').get('display_name'),
            request.user.display_name
        )
        self.assertEqual(
            response_json.get('file_type'),
            'DocumentFile'
        )
        self.assertIn('document_1.pdf', response_json.get('url'))

    @raises(FileTypeError)
    def test_create_document_and_respond_with_unsupported_type(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        document = File(open(
            normpath(join(
                dirname(abspath(__file__)),
                'files/document_2.doc'
            )),
            'rb'
        ))

        data = {
            'name': 'A test document',
            'description': 'Test document description',
            'file': document
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        view.create_and_respond(request, self.contribution)

    # This test does not work on Travis CI, but can be enabled to run locally.
    # Need to set settings.ENABLE_VIDEO = True and fill in YOUTUBE_* settings.
    # def test_create_video_and_respond(self):
    #     url = reverse(
    #         'api:project_media',
    #         kwargs={
    #             'project_id': self.project.id,
    #             'contribution_id': self.contribution.id
    #         }
    #     )
    #
    #     video_file = File(open(normpath(join(dirname(abspath(__file__)), 'files/video.MOV')), 'rb'))
    #
    #     data = {
    #         'name': 'A test video',
    #         'description': 'Test video description',
    #         'file': video_file
    #     }
    #
    #     request = self.factory.post(url, data)
    #     request.user = self.admin
    #     view = MediaAbstractAPIView()
    #     view.request = request
    #
    #     response = self.render(
    #         view.create_and_respond(request, self.contribution)
    #     )
    #
    #     response_json = json.loads(response.content)
    #     self.assertEqual(
    #         response_json.get('name'),
    #         data.get('name')
    #     )
    #     self.assertEqual(
    #         response_json.get('description'),
    #         data.get('description')
    #     )
    #     self.assertEqual(
    #         response_json.get('creator').get('display_name'),
    #         request.user.display_name
    #     )
    #     self.assertEqual(
    #         response_json.get('file_type'),
    #         'VideoFile'
    #     )
    #     self.assertIsNotNone(response_json.get('url'))

    def test_create_audio_and_respond(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        audio_file = File(open(
            normpath(join(
                dirname(abspath(__file__)),
                'files/audio_1.mp3'
            )),
            'rb'
        ))

        data = {
            'name': 'A test sound',
            'description': 'Test sound description',
            'file': audio_file
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        response = self.render(
            view.create_and_respond(request, self.contribution)
        )

        response_json = json.loads(response.content)
        self.assertEqual(
            response_json.get('name'),
            data.get('name')
        )
        self.assertEqual(
            response_json.get('description'),
            data.get('description')
        )
        self.assertEqual(
            response_json.get('creator').get('display_name'),
            request.user.display_name
        )
        self.assertEqual(
            response_json.get('file_type'),
            'AudioFile'
        )
        self.assertIn('audio_1.mp3', response_json.get('url'))

    def test_create_audio_and_convert_3gp(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        audio_file = File(open(
            normpath(join(
                dirname(abspath(__file__)),
                'files/audio_2.3gp'
            )),
            'rb'
        ))

        data = {
            'name': 'A test sound',
            'description': 'Test sound description',
            'file': audio_file
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        response = self.render(
            view.create_and_respond(request, self.contribution)
        )

        response_json = json.loads(response.content)
        self.assertEqual(
            response_json.get('name'),
            data.get('name')
        )
        self.assertEqual(
            response_json.get('description'),
            data.get('description')
        )
        self.assertEqual(
            response_json.get('creator').get('display_name'),
            request.user.display_name
        )
        self.assertEqual(
            response_json.get('file_type'),
            'AudioFile'
        )
        self.assertIn('audio_2.mp3', response_json.get('url'))

    def test_create_audio_and_convert_opus(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        audio_file = File(open(
            normpath(join(
                dirname(abspath(__file__)),
                'files/audio_1.opus'
            )),
            'rb'
        ))

        data = {
            'name': 'A test sound',
            'description': 'Test sound description (opus)',
            'file': audio_file
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        response = self.render(
            view.create_and_respond(request, self.contribution)
        )

        response_json = json.loads(response.content)
        self.assertEqual(
            response_json.get('name'),
            data.get('name')
        )
        self.assertEqual(
            response_json.get('description'),
            data.get('description')
        )
        self.assertEqual(
            response_json.get('creator').get('display_name'),
            request.user.display_name
        )
        self.assertEqual(
            response_json.get('file_type'),
            'AudioFile'
        )
        self.assertIn('audio_1.mp3', response_json.get('url'))

    def test_create_audio_and_convert_m4a(self):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        audio_file = File(open(
            normpath(join(
                dirname(abspath(__file__)),
                'files/audio_1.m4a'
            )),
            'rb'
        ))

        data = {
            'name': 'A test sound',
            'description': 'Test sound description (m4a)',
            'file': audio_file
        }

        request = self.factory.post(url, data)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        response = self.render(
            view.create_and_respond(request, self.contribution)
        )

        response_json = json.loads(response.content)
        self.assertEqual(
            response_json.get('name'),
            data.get('name')
        )
        self.assertEqual(
            response_json.get('description'),
            data.get('description')
        )
        self.assertEqual(
            response_json.get('creator').get('display_name'),
            request.user.display_name
        )
        self.assertEqual(
            response_json.get('file_type'),
            'AudioFile'
        )
        self.assertIn('audio_1.mp3', response_json.get('url'))


class MediaAbstractAPIViewTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserFactory.create()
        self.creator = UserFactory.create()
        self.project = ProjectFactory(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )

        self.contribution = ObservationFactory.create(
            **{'project': self.project, 'creator': self.creator}
        )

        self.image_file = ImageFileFactory.create(
            **{'contribution': self.contribution, 'creator': self.creator}
        )

    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/images/*'
        ))
        for f in files:
            os.remove(f)

    def render(self, response):
        response.accepted_renderer = JSONRenderer()
        response.accepted_media_type = 'application/json'
        response.renderer_context = {'blah': 'blubb'}
        return response.render()

    def test_get_single_and_respond(self):
        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': self.image_file.id
            }
        )

        request = self.factory.get(url)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        response = self.render(
            view.get_single_and_respond(
                request,
                self.image_file
            )
        )
        response_json = json.loads(response.content)
        self.assertEqual(response_json.get('id'), self.image_file.id)

    @raises(MediaFile.DoesNotExist)
    def test_delete_and_respond_with_admin(self):
        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': self.image_file.id
            }
        )

        request = self.factory.delete(url)
        request.user = self.admin
        view = MediaAbstractAPIView()
        view.request = request

        view.delete_and_respond(request, self.contribution, self.image_file)
        MediaFile.objects.get(pk=self.image_file.id)

    @raises(MediaFile.DoesNotExist)
    def test_delete_and_respond_with_contributor(self):
        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': self.image_file.id
            }
        )

        request = self.factory.delete(url)
        request.user = self.creator
        view = MediaAbstractAPIView()
        view.request = request

        view.delete_and_respond(request, self.contribution, self.image_file)
        MediaFile.objects.get(pk=self.image_file.id)

    @raises(PermissionDenied)
    def test_delete_and_respond_with_some_dude(self):
        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': self.image_file.id
            }
        )

        request = self.factory.delete(url)
        request.user = UserFactory.create()
        view = MediaAbstractAPIView()
        view.request = request

        view.delete_and_respond(request, self.contribution, self.image_file)


class MediaAPIViewTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserFactory.create()
        self.creator = UserFactory.create()
        self.viewer = UserFactory.create()
        self.project = ProjectFactory(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )

        self.contribution = ObservationFactory.create(
            **{'project': self.project, 'creator': self.creator}
        )

        ImageFileFactory.create_batch(5, **{'contribution': self.contribution})

    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/images/*'
        ))
        for f in files:
            os.remove(f)

    def get(self, user):
        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        request = self.factory.get(url)
        force_authenticate(request, user)
        view = MediaAPIView.as_view()
        return view(
            request,
            project_id=self.project.id,
            contribution_id=self.contribution.id
        ).render()

    def post(self, user, data=None):
        if user.is_anonymous and not User.objects.filter(
                display_name='AnonymousUser').exists():
            UserFactory.create(display_name='AnonymousUser')

        if data is None:
            data = {
                'name': 'A test image',
                'description': 'Test image description',
                'file': get_image()
            }

        url = reverse(
            'api:project_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id
            }
        )

        request = self.factory.post(url, data)
        force_authenticate(request, user)
        view = MediaAPIView.as_view()
        return view(
            request,
            project_id=self.project.id,
            contribution_id=self.contribution.id
        ).render()

    def test_get_images_with_admin(self):
        response = self.get(self.admin)
        self.assertEqual(response.status_code, 200)

    def test_get_images_with_contributor(self):
        response = self.get(self.creator)
        self.assertEqual(response.status_code, 200)

    def test_get_images_with_some_dude(self):
        response = self.get(UserFactory.create())
        self.assertEqual(response.status_code, 404)

    def test_get_images_with_anonymous(self):
        response = self.get(AnonymousUser())
        self.assertEqual(response.status_code, 404)

    def test_upload_image_with_admin(self):
        response = self.post(self.admin)
        self.assertEqual(response.status_code, 201)

    def test_upload_image_with_contributor(self):
        response = self.post(self.creator)
        self.assertEqual(response.status_code, 201)

    def test_upload_image_with_some_dude(self):
        response = self.post(UserFactory.create())
        self.assertEqual(response.status_code, 404)

    def test_upload_image_with_anonymous(self):
        response = self.post(AnonymousUser())
        self.assertEqual(response.status_code, 404)

    def test_post_images_with_anonymous_to_public(self):
        self.project.isprivate = False
        self.project.everyone_contributes = 'true'
        self.project.save()

        response = self.post(AnonymousUser())
        self.assertEqual(response.status_code, 201)

    def test_upload_unsupported_file_format(self):
        xyz_file = BytesIO()
        xyz = Image.new('RGBA', size=(50, 50), color=(256, 0, 0))
        xyz.save(xyz_file, 'png')
        xyz_file.seek(0)

        data = {
            'name': 'A test image',
            'description': 'Test image description',
            'file': ContentFile(xyz_file.read(), 'test.xyz')
        }

        response = self.post(self.admin, data=data)
        self.assertEqual(response.status_code, 400)

    def test_upload_with_loooooong_filename(self):
        data = {
            'name': 'A test image ',
            'description': 'Test image description',
            'file': get_image(file_name='One two three four six seven eight '
                                        'nine ten eleven twelve thirteen '
                                        'fourteen fifteen.png')
        }

        response = self.post(self.admin, data=data)
        self.assertEqual(response.status_code, 201)


class AllContributionsSingleMediaApiViewTest(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserFactory.create()
        self.creator = UserFactory.create()
        self.viewer = UserFactory.create()
        self.project = ProjectFactory(
            add_admins=[self.admin],
            add_contributors=[self.creator]
        )

        self.contribution = ObservationFactory.create(
            **{'project': self.project, 'creator': self.creator}
        )

        self.image_file = ImageFileFactory.create(
            **{'contribution': self.contribution, 'creator': self.creator}
        )

    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/images/*'
        ))
        for f in files:
            os.remove(f)

    def get(self, user):
        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': self.image_file.id
            }
        )

        request = self.factory.get(url)
        force_authenticate(request, user)
        view = SingleMediaAPIView.as_view()
        return view(
            request,
            project_id=self.project.id,
            contribution_id=self.contribution.id,
            file_id=self.image_file.id
        ).render()

    def delete(self, user, image_id=None):
        if image_id is None:
            image_id = self.image_file.id

        url = reverse(
            'api:project_single_media',
            kwargs={
                'project_id': self.project.id,
                'contribution_id': self.contribution.id,
                'file_id': image_id
            }
        )

        request = self.factory.delete(url)
        force_authenticate(request, user)
        view = SingleMediaAPIView.as_view()
        return view(
            request,
            project_id=self.project.id,
            contribution_id=self.contribution.id,
            file_id=image_id
        ).render()

    def test_get_image_with_admin(self):
        response = self.get(self.admin)
        self.assertEqual(response.status_code, 200)

    def test_get_image_with_contributor(self):
        response = self.get(self.creator)
        self.assertEqual(response.status_code, 200)

    def test_get_image_with_some_dude(self):
        response = self.get(UserFactory.create())
        self.assertEqual(response.status_code, 404)

    def test_get_image_with_anonymous(self):
        response = self.get(AnonymousUser())
        self.assertEqual(response.status_code, 404)

    def test_delete_image_with_admin(self):
        response = self.delete(self.admin)
        self.assertEqual(response.status_code, 204)

    def test_get_not_existing_image_with_admin(self):
        response = self.delete(self.admin, image_id=545487654)
        self.assertEqual(response.status_code, 404)

    def test_delete_image_with_contributor(self):
        response = self.delete(self.creator)
        self.assertEqual(response.status_code, 204)

    def test_delete_image_with_some_dude(self):
        response = self.delete(UserFactory.create())
        self.assertEqual(response.status_code, 404)

    def test_delete_image_with_anonymous(self):
        response = self.delete(AnonymousUser())
        self.assertEqual(response.status_code, 404)



"""Custom placeholder template filters"""

from django import template

register = template.Library()


@register.filter()
def project_replace(value, project_name):
    return value.replace('$project$', hashify(project_name))


@register.filter()
def hashify(value):
    return "#" + value.replace(' ', '').replace('-', '').lower()


@register.filter()
def add_link(value, link):
    link = '<span id= "breakable" >' + link + '</span>'
    value = value.replace("$link$", link)
    return value



"""Tests for core views."""

import json

from django.test import TestCase
from django.http import HttpRequest
from django.contrib.auth.models import AnonymousUser
from django.contrib.messages.storage.fallback import FallbackStorage
from django.template.loader import render_to_string
from django.contrib.sites.shortcuts import get_current_site

from geokey.version import get_version
from geokey import version
from geokey.core.views import InfoAPIView, LoggerList
from geokey.extensions.base import register, deregister
from geokey.users.tests.model_factories import UserFactory
from geokey.projects.tests.model_factories import ProjectFactory
from geokey.core.models import LoggerHistory


# ############################################################################
#
# PUBLIC API
#
# ############################################################################

class InfoAPIViewTest(TestCase):
    """Test public API for GeoKey server information."""

    def setUp(self):
        """Set up test."""
        self.view = InfoAPIView.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'

        register('A', 'A', display_admin=True, superuser=False, version='1.0')
        register('B', 'B', display_admin=True, superuser=False)
        register('S', 'S', display_admin=True, superuser=True, version='0.1.0')

    def tearDown(self):
        """Tear down test."""
        deregister('A')
        deregister('B')
        deregister('S')

    def contains_extension(self, ext_id, installed_extensions):
        """Helper to check if extension is in installed extensions."""
        for extension in installed_extensions:
            if 'name' in extension and extension['name'] == ext_id:
                return True

        return False

    def test_get(self):
        """Test GET."""
        self.request.user = AnonymousUser()
        response = self.view(self.request).render()

        self.assertEqual(response.status_code, 200)

        response = json.loads(response.content)
        self.assertIn('geokey', response)

        geokey = response.get('geokey')
        self.assertEqual(geokey.get('version'), get_version())
        self.assertIn('installed_extensions', geokey)

        installed_extensions = geokey.get('installed_extensions')
        self.assertTrue(self.contains_extension('A', installed_extensions))
        self.assertTrue(self.contains_extension('B', installed_extensions))
        self.assertTrue(self.contains_extension('S', installed_extensions))


# ############################################################################
#
# ADMIN PAGES
#
# ############################################################################


class LoggerListTest(TestCase):
    """Test a list of logs page."""

    def setUp(self):
        """Set up tests."""
        self.view = LoggerList.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = AnonymousUser()

        self.project = ProjectFactory.create()

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser should redirect to the login
        page
        """

        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

    def test_get_with_user(self):
        """
        Accessing the view with normal user should render the page with an
        error message
        """

        user = UserFactory.create()
        project = ProjectFactory.create()

        self.request.user = user
        response = self.view(self.request, project_id=project.id).render()

        rendered = render_to_string(
            'logger/logger_list.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_get_with_admin(self):
        """
        Accessing the view with project admin.
        It should render the page.
        """
        project = ProjectFactory.create()
        user = project.creator

        logs = LoggerHistory.objects.filter(
            project__contains={'id': str(project.id)})

        logger_list = LoggerList()

        self.request.user = user
        response = self.view(self.request, project_id=project.id).render()

        rendered = render_to_string(
            'logger/logger_list.html',
            {
                'project': project,
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version(),
                'logs': logger_list.paginate_logs(
                    logs[::-1],
                    self.request.GET.get('page'))
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)



"""Test for utils."""

from django.test import TestCase
from django.utils import timezone

from allauth.socialaccount.models import SocialAccount, SocialApp

from geokey.socialinteractions.utils import (
    start2pull,
    check_dates,
    create_new_observation,
    get_category_and_field,
    pull_from_social_media
)
from geokey.categories.tests.model_factories import CategoryFactory, TextFieldFactory, NumericFieldFactory

from geokey.users.tests.model_factories import UserFactory
from geokey.projects.tests.model_factories import ProjectFactory
from geokey.socialinteractions.tests.model_factories import SocialInteractionPullFactory
from geokey.socialinteractions.models import get_ready_to_post
from geokey.contributions.models import Observation
from geokey.contributions.tests.model_factories import ObservationFactory

from datetime import timedelta, datetime


class CheckDatesTest(TestCase):
    """Test for 'check_dates'."""

    def test_method(self):
        """Test method."""
        now = timezone.now()
        updated_at = now - timedelta(minutes=10)

        value_true = check_dates(updated_at, '5min')

        self.assertEqual(True, value_true)

        value_false = check_dates(updated_at, 'daily')

        self.assertNotEqual(True, value_false)


class GetCategoryAndFieldTest(TestCase):
    """Test for 'get_category_and_field'."""

    def setUp(self):
        """Set up for test method get_category_and_field."""
        self.admin = UserFactory.create()
        self.project = ProjectFactory.create(creator=self.admin)

        self.socialaccount = SocialAccount.objects.create(
            user=self.admin, provider='facebook', uid='1')
        self.category = CategoryFactory.create(
            name='Tweets',
            creator=self.admin,
            project=self.project
        )

        self.field_text = TextFieldFactory.create(
            key='tweet',
            category=self.category
        )

        self.tweet_id_field = NumericFieldFactory.create(
            key='tweet-id',
            category=self.category
        )

    def test_method_when_check_when_category_and_field_exists(self):
        """Check method when category and field exists."""
        tweet_cat, text_field, tweet_id_field = get_category_and_field(
            self.project,
            self.socialaccount)

        self.assertEqual(tweet_cat.id, self.category.id)
        self.assertEqual(tweet_cat.name, self.category.name)

        self.assertEqual(text_field.id, self.field_text.id)
        self.assertEqual(text_field.key, self.field_text.key)

        self.assertEqual(tweet_id_field.id, self.tweet_id_field.id)
        self.assertEqual(tweet_id_field.key, self.tweet_id_field.key)

    def test_method_when_check_when_category_and_field_does_not_exist(self):
        """Check method when category does not exist and field exists."""
        self.category.delete()
        self.category.save()

        tweet_cat, text_field, tweet_id_field = get_category_and_field(
            self.project,
            self.socialaccount)

        self.assertNotEqual(tweet_cat.id, self.category.id)
        self.assertEqual(tweet_cat.name, self.category.name)

        self.assertNotEqual(text_field.id, self.field_text.id)
        self.assertEqual(text_field.key, self.field_text.key)

        self.assertNotEqual(tweet_id_field.id, self.tweet_id_field.id)
        self.assertEqual(tweet_id_field.key, self.tweet_id_field.key)


class CreateNewObservationTest(TestCase):
    """Test for method 'create_new_observation'."""

    def setUp(self):
        """Set up test method 'create_new_observation'."""
        self.admin = UserFactory.create()
        self.project = ProjectFactory.create(creator=self.admin)

        self.socialaccount = SocialAccount.objects.create(
            user=self.admin, provider='facebook', uid='1')
        self.category = CategoryFactory.create(
            name='Tweets',
            creator=self.admin,
            project=self.project
        )

        self.field_text = TextFieldFactory.create(
            key='tweet',
            category=self.category
        )

        self.tweet_id_field = NumericFieldFactory.create(
            key='tweet-id',
            category=self.category
        )

        self.si_pull = SocialInteractionPullFactory.create(
            socialaccount=self.socialaccount,
            project=self.project,
            creator=self.admin)

        self.geo_tweet = {
            'geometry': u'POINT (-0.1350858 51.5246635)',
            'text': u'#Project2 scorpion @adeuonce',
            'created_at': datetime(2017, 5, 23, 14, 43, 1),
            'id': 867028097530572801,
            'user': u'Pepito Grillo'}

    def test_method_create_new_observation(self):
        """Test for method 'create_new_observation'."""
        init_obs = Observation.objects.count()

        create_new_observation(
            self.si_pull,
            self.geo_tweet,
            self.category,
            self.field_text,
            self.tweet_id_field
        )

        self.assertEqual(init_obs + 1, Observation.objects.count())


class PullFromSocialMediaTest(TestCase):
    """Test for pull_from_social_media."""

    def setUp(self):
        """Setup for test."""
        self.provider = 'twitter'
        self.text_to_pull = '#Project2'
        self.since_id = 928637049644646400
        self.admin_user = UserFactory.create()
        self.app = SocialApp.objects.create(
            provider='twitter',
            name='Twitter',
            client_id='xxxxxxxxxxxxxxxxxx',
            secret='xxxxxxxxxxxxxxxxxx',
            key=''
        )
        self.socialaccount = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='1')

    def test_pull_from_social_media_with_fake_app(self):
        """Test pull_from_social_media with fake app."""
        access_token = 'ahahashasgasgasfgas'
        value = pull_from_social_media(
            self.provider,
            access_token,
            self.text_to_pull,
            self.since_id,
            self.app)
        self.assertEqual(value, "You are not authenticated")


class GetReadyToPostTest(TestCase):
    """Test for get_ready_to_post."""

    def setUp(self):
        """Setup for test."""
        self.user = UserFactory.create()
        self.category_tweet = CategoryFactory.create(
            name="Tweets")

        self.observation = ObservationFactory.create()
        self.observation_tweet = ObservationFactory.create(
            category=self.category_tweet)

    def test_method_with_regular_category(self):
        """Test method with regular category."""
        value = get_ready_to_post(self.observation)

        self.assertEqual(value, "posted to social media")

    def test_method_with_category_name_tweet(self):
        """Test method with category name as 'Tweets'."""
        value = get_ready_to_post(self.observation_tweet)

        self.assertEqual(value, "Category name is Tweets")



# -*- coding: utf-8 -*-


from django.db import migrations
try:
    from django.contrib.postgres.fields import JSONField
except ImportError:
    from django_pgjson.fields import JsonBField as JSONField


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0005_auto_20150202_1135'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalobservation',
            name='properties',
            field=JSONField(default={}),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='observation',
            name='properties',
            field=JSONField(default={}),
            preserve_default=True,
        ),
    ]



"""Base for categories."""

from model_utils import Choices

STATUS = Choices('active', 'inactive')

FREQUENCY = Choices('5min',
                    '10min',
                    '20min',
                    '30min',
                    'hourly',
                    'daily',
                    'weekly',
                    'fortnightly',
                    'monthly')

freq_dic = {
    '5min': 0.083,
    '10min': 0.17,
    '20min': 0.33,
    '30min': 0.5,
    'hourly': 1,
    'daily': 24,
    'weekly': 168,
    'fortnightly': 336,
    'monthly': 672
}



"""Tests for AJAX views of users."""

import json

from django.test import TestCase
from django.core.urlresolvers import reverse

from rest_framework.test import APIRequestFactory, force_authenticate
from rest_framework import status

from geokey.projects.tests.model_factories import ProjectFactory

from .model_factories import UserFactory, UserGroupFactory
from ..views import (
    UserGroup, UserGroupUsers, UserGroupSingleUser,
)
from ..models import UserGroup as Group


class QueryUsersTest(TestCase):

    def _get(self, query):
        return self.client.get('/ajax/users/?query=' + query)

    def setUp(self):
        UserFactory.create(**{
            'display_name': 'Peter Schmeichel'
        })
        UserFactory.create(**{
            'display_name': 'George Best'
        })
        UserFactory.create(**{
            'display_name': 'Luis Figo'
        })
        UserFactory.create(**{
            'display_name': 'pete23'
        })
        UserFactory.create(**{
            'display_name': 'pet48'
        })
        UserFactory.create(**{
            'display_name': 'Frank Lampard'
        })

    def test_query_pet(self):
        response = self._get('pet')

        result_set = json.loads(response.content)
        self.assertEqual(len(result_set), 3)
        self.assertContains(response, 'Schmeichel')
        self.assertContains(response, 'pete23')
        self.assertContains(response, 'pet48')

    def test_query_peter(self):
        response = self._get('peter')
        result_set = json.loads(response.content)
        self.assertEqual(len(result_set), 1)
        self.assertContains(response, 'Schmeichel')

    def test_query_anonymous(self):
        response = self._get('anon')
        result_set = json.loads(response.content)
        self.assertEqual(len(result_set), 0)

    def test_no_query(self):
        response = self.client.get('/ajax/users/')
        result_set = json.loads(response.content)
        self.assertEqual(len(result_set), 0)


class UserGroupTest(TestCase):

    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserFactory.create()
        self.contributor = UserFactory.create()
        self.non_member = UserFactory.create()
        self.user_to_add = UserFactory.create()

        self.project = ProjectFactory.create(
            add_admins=[self.admin]
        )

        self.contributors = UserGroupFactory(
            add_users=[self.contributor],
            **{'project': self.project}
        )

    def put(self, user, data):
        url = reverse('ajax:usergroup', kwargs={
            'project_id': self.project.id,
            'usergroup_id': self.contributors.id
        })
        request = self.factory.put(
            url, json.dumps(data), content_type='application/json')
        force_authenticate(request, user=user)
        view = UserGroup.as_view()

        return view(
            request,
            project_id=self.project.id,
            usergroup_id=self.contributors.id).render()

    def test_update_with_admin(self):
        response = self.put(self.admin, {'can_contribute': False})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertFalse(
            Group.objects.get(pk=self.contributors.id).can_contribute,
        )

    def test_when_project_is_locked(self):
        self.project.islocked = True
        self.project.save()

        response = self.put(self.admin, {'can_contribute': False})
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertTrue(
            Group.objects.get(pk=self.contributors.id).can_contribute,
        )

    def test_invalid_update_with_admin(self):
        response = self.put(self.admin, {'can_contribute': 'Blah'})
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertTrue(
            Group.objects.get(pk=self.contributors.id).can_contribute,
        )

    def test_update_description_with_contributor(self):
        response = self.put(self.contributor, {'can_contribute': False})
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertTrue(
            Group.objects.get(pk=self.contributors.id).can_contribute,
        )

    def test_update_description_with_non_member(self):
        response = self.put(self.non_member, {'can_contribute': False})
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertTrue(
            Group.objects.get(pk=self.contributors.id).can_contribute,
        )


class UserGroupUsersTest(TestCase):

    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserFactory.create()
        self.contributor = UserFactory.create()
        self.non_member = UserFactory.create()
        self.user_to_add = UserFactory.create()

        self.project = ProjectFactory.create(
            add_admins=[self.admin]
        )

        self.contributors = UserGroupFactory(
            add_users=[self.contributor],
            **{'project': self.project}
        )

    def test_add_to_not_existing_usergroup(self):
        request = self.factory.post(
            '/ajax/projects/%s/usergroups/%s/users/' %
            (self.project.id, 6545454844545648),
            {'user_id': self.user_to_add.id}
        )
        force_authenticate(request, user=self.admin)
        view = UserGroupUsers.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=6545454844545648
        ).render()

        self.assertEqual(response.status_code, 404)

    def test_add_not_existing_user(self):
        request = self.factory.post(
            '/ajax/projects/%s/usergroups/%s/users/' %
            (self.project.id, self.contributors.id),
            {'user_id': 4445468756454}
        )
        force_authenticate(request, user=self.admin)
        view = UserGroupUsers.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=self.contributors.id
        ).render()

        self.assertEqual(response.status_code, 400)

    def test_add_on_locked_project(self):
        self.project.islocked = True
        self.project.save()

        request = self.factory.post(
            '/ajax/projects/%s/usergroups/%s/users/' %
            (self.project.id, self.contributors.id),
            {'user_id': self.user_to_add.id}
        )
        force_authenticate(request, user=self.admin)
        view = UserGroupUsers.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=self.contributors.id
        ).render()

        self.assertEqual(response.status_code, 400)

    def test_add_contributor_with_admin(self):
        request = self.factory.post(
            '/ajax/projects/%s/usergroups/%s/users/' %
            (self.project.id, self.contributors.id),
            {'user_id': self.user_to_add.id}
        )
        force_authenticate(request, user=self.admin)
        view = UserGroupUsers.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=self.contributors.id
        ).render()

        self.assertEqual(response.status_code, 201)
        self.assertIn(
            self.user_to_add,
            self.contributors.users.all()
        )

    def test_add_contributor_with_contributor(self):
        request = self.factory.post(
            '/ajax/projects/%s/usergroups/%s/users/' %
            (self.project.id, self.contributors.id),
            {'user_id': self.user_to_add.id}
        )
        force_authenticate(request, user=self.contributor)
        view = UserGroupUsers.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=self.contributors.id
        ).render()

        self.assertEqual(response.status_code, 403)
        self.assertNotIn(
            self.user_to_add,
            self.contributors.users.all()
        )

    def test_add_contributor_with_non_member(self):
        request = self.factory.post(
            '/ajax/projects/%s/usergroups/%s/users/' %
            (self.project.id, self.contributors.id),
            {'user_id': self.user_to_add.id}
        )
        force_authenticate(request, user=self.non_member)
        view = UserGroupUsers.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=self.contributors.id
        ).render()

        self.assertEqual(response.status_code, 404)
        self.assertNotIn(
            self.user_to_add,
            self.contributors.users.all()
        )


class UserGroupSingleUserTest(TestCase):

    def setUp(self):
        self.factory = APIRequestFactory()
        self.admin = UserFactory.create()
        self.contributor = UserFactory.create()
        self.non_member = UserFactory.create()
        self.contrib_to_remove = UserFactory.create()

        self.project = ProjectFactory.create(
            add_admins=[self.admin]
        )

        self.contributors = UserGroupFactory(
            add_users=[self.contributor, self.contrib_to_remove],
            **{'project': self.project, 'can_contribute': True}
        )

    def test_delete_not_existing_user(self):
        user = UserFactory.create()
        request = self.factory.delete(
            '/ajax/projects/%s/usergroups/%s/users/%s/' %
            (self.project.id, self.contributors.id, user.id),
        )
        force_authenticate(request, user=self.admin)
        view = UserGroupSingleUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=self.contributors.id,
            user_id=user.id
        ).render()

        self.assertEqual(response.status_code, 404)

    def test_delete_from_not_existing_usergroup(self):
        request = self.factory.delete(
            '/ajax/projects/%s/usergroups/%s/users/%s/' %
            (self.project.id, 455646445484545, self.contrib_to_remove.id),
        )
        force_authenticate(request, user=self.admin)
        view = UserGroupSingleUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=455646445484545,
            user_id=self.contrib_to_remove.id
        ).render()

        self.assertEqual(response.status_code, 404)

    def test_delete_contributor_with_admin(self):
        request = self.factory.delete(
            '/ajax/projects/%s/usergroups/%s/users/%s/' %
            (self.project.id, self.contributors.id, self.contrib_to_remove.id),
        )
        force_authenticate(request, user=self.admin)
        view = UserGroupSingleUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=self.contributors.id,
            user_id=self.contrib_to_remove.id
        ).render()

        self.assertEqual(response.status_code, 204)
        self.assertNotIn(
            self.contrib_to_remove,
            self.contributors.users.all()
        )

    def test_delete_contributor_with_contributor(self):
        request = self.factory.delete(
            '/ajax/projects/%s/usergroups/%s/users/%s/' %
            (self.project.id, self.contributors.id, self.contrib_to_remove.id)
        )
        force_authenticate(request, user=self.contributor)
        view = UserGroupSingleUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=self.contributors.id,
            user_id=self.contrib_to_remove.id
        ).render()

        self.assertEqual(response.status_code, 403)
        self.assertIn(
            self.contrib_to_remove,
            self.contributors.users.all()
        )

    def test_delete_contributor_with_non_member(self):
        request = self.factory.delete(
            '/ajax/projects/%s/usergroups/%s/users/%s/' %
            (self.project.id, self.contributors.id, self.contrib_to_remove.id)
        )
        force_authenticate(request, user=self.non_member)
        view = UserGroupSingleUser.as_view()
        response = view(
            request,
            project_id=self.project.id,
            usergroup_id=self.contributors.id,
            user_id=self.contrib_to_remove.id
        ).render()
        self.assertEqual(response.status_code, 404)
        self.assertIn(
            self.contrib_to_remove,
            self.contributors.users.all()
        )



"""Utils for contributions

This code is base on upload_video sample provided by google here:
 https://github.com/youtube/api-samples/blob/master/python/upload_video.py.
"""


import os

from apiclient.discovery import build
from oauth2client import _helpers
from oauth2client.file import Storage
from apiclient.http import MediaFileUpload
from apiclient.errors import HttpError
from oauth2client.client import OAuth2WebServerFlow

from argparse import Namespace

import httplib2
from six.moves import http_client

from django.conf import settings


YOUTUBE_API_SERVICE_NAME = "youtube"
YOUTUBE_API_VERSION = "v3"
RETRIABLE_STATUS_CODES = [500, 502, 503, 504]
RETRIABLE_EXCEPTIONS = (
    httplib2.HttpLib2Error, IOError, http_client.NotConnected,
    http_client.IncompleteRead, http_client.ImproperConnectionState,
    http_client.CannotSendRequest, http_client.CannotSendHeader,
    http_client.ResponseNotReady, http_client.BadStatusLine
)


@_helpers.positional(2)
def my_flow_from_clientsecrets(client_info, scope, redirect_uri=None,
    message=None, cache=None, login_hint=None, device_uri=None, pkce=None,
    code_verifier=None):
    """Create a Flow from a clientsecrets file.

    Will create the right kind of Flow based on the contents of the
    clientsecrets file or will raise InvalidClientSecretsError for unknown
    types of Flows.

    Args:
        filename: string, File name of client secrets.
        scope: string or iterable of strings, scope(s) to request.
        redirect_uri: string, Either the string 'urn:ietf:wg:oauth:2.0:oob' for
                      a non-web-based application, or a URI that handles the
                      callback from the authorization server.
        message: string, A friendly string to display to the user if the
                 clientsecrets file is missing or invalid. If message is
                 provided then sys.exit will be called in the case of an error.
                 If message in not provided then
                 clientsecrets.InvalidClientSecretsError will be raised.
        cache: An optional cache service client that implements get() and set()
               methods. See clientsecrets.loadfile() for details.
        login_hint: string, Either an email address or domain. Passing this
                    hint will either pre-fill the email box on the sign-in form
                    or select the proper multi-login session, thereby
                    simplifying the login flow.
        device_uri: string, URI for device authorization endpoint. For
                    convenience defaults to Google's endpoints but any
                    OAuth 2.0 provider can be used.

    Returns:
        A Flow object.

    Raises:
        UnknownClientSecretsFlowError: if the file describes an unknown kind of
                                       Flow.
        clientsecrets.InvalidClientSecretsError: if the clientsecrets file is
                                                 invalid.
    """
    try:
        constructor_kwargs = {
            'redirect_uri': redirect_uri,
            'auth_uri': client_info['auth_uri'],
            'token_uri': client_info['token_uri'],
            'login_hint': login_hint,
        }
        revoke_uri = client_info.get('revoke_uri')
        optional = ('revoke_uri', 'device_uri', 'pkce', 'code_verifier')
        for param in optional:
            if locals()[param] is not None:
                constructor_kwargs[param] = locals()[param]
        return OAuth2WebServerFlow(
            client_info['client_id'], client_info['client_secret'],
            scope, **constructor_kwargs)
    except Exception as e:
        print(("error", e))


def resumable_upload(insert_request):
    """Start and check the uploaded request. """
    response = None
    error = None
    while response is None:
        try:
            print("Uploading file...")
            status, response = insert_request.next_chunk()
            if response is not None:
                if 'id' in response:
                    print(("Video id '%s' was successfully uploaded." %response['id']))
                    return response['id']
            else:
                exit("The upload failed with an unexpected response: %s" % response)
        except HttpError as e:
            if e.resp.status in RETRIABLE_STATUS_CODES:
                error = "A retriable HTTP error %d occurred:\n%s" % (e.resp.status, e.content)
            else:
                raise


def initialize_upload(youtube, options):
    """Initilize the upload video."""
    tags = None
    if options.keywords:
        tags = options.keywords.split(",")

    body = dict(
        snippet=dict(
            title=options.title,
            description=options.description,
            tags=tags,
            categoryId=options.category),
        status=dict(
            privacyStatus=options.privacyStatus))

    # Call the API's videos.insert method to create and upload the video.
    insert_request = youtube.videos().insert(
        part=",".join(list(body.keys())),
        body=body,
        media_body=MediaFileUpload(options.file, chunksize=-1, resumable=True))

    return resumable_upload(insert_request)


def get_args(name, path):
    """Create arguments to passed to the ."""
    args = Namespace(
        auth_host_name='localhost',
        auth_host_port=[8080, 8000],
        category='22',
        description='Test Description',
        file=path,
        keywords='',
        logging_level='ERROR',
        noauth_local_webserver=True,
        privacyStatus='public',
        title=name)

    return args


def get_authenticated_service():
    """Get youtube credentials identified. """
    try:
        youtube_uploader = settings.YOUTUBE_UPLOADER
    except Exception as e:
        raise ValueError(e)

    scope = youtube_uploader['scope']
    credentials_path = youtube_uploader['credentials_path']
    client_info = youtube_uploader['client_info']
    credentials_file = youtube_uploader['credentials_file']

    my_flow_from_clientsecrets(client_info, scope)
    module_dir = os.path.abspath(credentials_path)
    file_path = os.path.join(module_dir, credentials_file)
    storage = Storage(file_path)
    credentials = storage.get()

    try:
        builded = build(YOUTUBE_API_SERVICE_NAME, YOUTUBE_API_VERSION,
                        http=credentials.authorize(httplib2.Http()))
        return builded
    except Exception as e:
        print(("error", e))



"""Base for projects."""

from model_utils import Choices

from django.conf import settings


STATUS = Choices('active', 'inactive', 'deleted')
allowed = ("auth", "false", "true")
if hasattr(settings, 'ALLOWED_CONTRIBUTORS'):
    allowed = settings.ALLOWED_CONTRIBUTORS
EVERYONE_CONTRIBUTES = Choices(*allowed)



"""Test all template tags."""

from django.test import TestCase

from geokey.core.models import LoggerHistory

from geokey.core.templatetags import logger


class TemplateTagsTest(TestCase):
    """Tests for template tags."""

    def test_check_media_file_type(self):
        """Test check the field type."""
        logger_1 = LoggerHistory.objects.create(
            user={'display_name': 'User', 'id': '5'},
            project={'name': 'Porject', 'id': '5'},
            category={'name': 'Category', 'id': '5'},
            field={'name': 'Field', 'id': '5', 'type': 'TextField'},
            action={'id': 'created', 'class': 'Field'}
        )

        field_type_1 = logger.check_field_type(logger_1.field.get('type'))
        self.assertTrue(field_type_1, 'Text field')

        logger_2 = logger_1
        logger_2.field = {'name': 'Field22', 'id': '5', 'type': 'TextField'}
        logger_2.save()
        field_type_2 = logger.check_field_type(logger_2.field.get('type'))
        self.assertTrue(field_type_2, 'Numeric field')

        logger_3 = logger_1
        logger_3.field = {'name': 'Field', 'id': '5', 'type': 'DateField'}
        logger_3.save()
        field_type_3 = logger.check_field_type(logger_3.field.get('type'))
        self.assertTrue(field_type_3, 'Date field')

        logger_4 = logger_1
        logger_4.field = {'name': 'Field', 'id': '5', 'type': 'DateTimeField'}
        logger_4.save()
        field_type_4 = logger.check_field_type(logger_4.field.get('type'))
        self.assertTrue(field_type_4, 'Date & time field')

        logger_5 = logger_1
        logger_5.field = {'name': 'Field', 'id': '5', 'type': 'LookupField'}
        logger_5.save()
        field_type_5 = logger.check_field_type(logger_5.field.get('type'))
        self.assertTrue(field_type_5, 'Select box field')

        logger_6 = logger_1
        logger_6.field = {'name': 'Field', 'id': '5', 'type': 'TimeField'}
        logger_6.save()
        field_type_6 = logger.check_field_type(logger_6.field.get('type'))
        self.assertTrue(field_type_6, 'Time field')

        logger_7 = logger_1
        logger_7.field = {
            'name': 'Field',
            'id': '5',
            'type': 'MultipleLookupField'}
        logger_7.save()
        field_type_7 = logger.check_field_type(logger_7.field.get('type'))
        self.assertTrue(field_type_7, 'Multiple select field')

    def test_check_field_type(self):
        """Test check the field type."""
        logger_11 = LoggerHistory.objects.create(
            user={'display_name': 'User', 'id': '5'},
            project={'name': 'Porject', 'id': '5'},
            category={'name': 'Category', 'id': '5'},
            mediafile={'name': 'MediaFile', 'id': '43', 'type': 'AudioFile'},
            action={'id': 'created', 'class': 'MediaFile'}
        )

        file_type_11 = logger.check_media_file_type(
            logger_11.mediafile.get('type'))
        self.assertTrue(file_type_11, 'Audio file')

        logger_22 = logger_11
        logger_22.mediafile = {
            'name': 'MediaFile',
            'id': '77',
            'type': 'VideoFile'}
        logger_22.save()
        file_type_22 = logger.check_media_file_type(
            logger_22.mediafile.get('type'))
        self.assertTrue(file_type_22, 'Video file')

        logger_33 = logger_11
        logger_33.mediafile = {
            'name': 'MediaFile',
            'id': '66',
            'type': 'ImageFile'}
        logger_33.save()
        file_type_33 = logger.check_media_file_type(
            logger_33.mediafile.get('type'))
        self.assertTrue(file_type_33, 'Image file')



"""Models for subsets."""

from django.conf import settings
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver

from allauth.socialaccount.models import SocialAccount, SocialToken, SocialApp

import tweepy
import facebook

from geokey.subsets.models import Subset
from geokey.contributions.models import Observation, Comment
from geokey.projects.models import Project
from geokey.core.signals import delete_project
from .base import STATUS, FREQUENCY


class SocialInteractionPost(models.Model):
    """Stores post social interaction."""

    created_at = models.DateTimeField(auto_now_add=True)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL)
    project = models.ForeignKey(
        'projects.Project',
        related_name='socialinteractions_post'
    )
    socialaccount = models.ForeignKey(
        SocialAccount,
        related_name='socialinteractions_post'
    )
    text_to_post = models.TextField(blank=True, null=True)
    link = models.TextField(blank=True, null=True)
    status = models.CharField(
        choices=STATUS,
        default=STATUS.active,
        max_length=20
    )


class SocialInteractionPull(models.Model):
    """Stores pull social interaction."""
    project = models.ForeignKey(
        'projects.Project',
        related_name='socialinteractions_pull'
    )
    created_at = models.DateTimeField(auto_now_add=True)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL)
    socialaccount = models.ForeignKey(
        SocialAccount,
        related_name='socialinteractions_pull'
    )
    text_to_pull = models.TextField(blank=True, null=True)
    frequency = models.CharField(
        choices=FREQUENCY,
        default=FREQUENCY.daily,
        max_length=20
    )
    status = models.CharField(
        choices=STATUS,
        default=STATUS.active,
        max_length=20
    )
    since_id = models.TextField(blank=True, null=True)
    updated_at = models.DateTimeField(null=True, auto_now_add=False)
    checked_at = models.DateTimeField(null=True, auto_now_add=False)


@receiver(delete_project)
def on_delete_project(project, **kwargs):
    """
    Receiver that is called after a project is deleted.

    Removes Socialinteractions.
    """

    try:
        SocialInteractionPull.objects.filter(project=project).delete()
        SocialInteractionPost.objects.filter(project=project).delete()
    except SocialInteractionPull.DoesNotExist:
        pass
    except SocialInteractionPost.DoesNotExist:
        pass


@receiver(post_save, sender=Observation)
def post_save_observation(sender, instance, created, **kwargs):
    """Signal when new Observation object is created."""
    if created:
        get_ready_to_post(instance)


def get_ready_to_post(instance):
    """Post/tweet to social media when a new Observation is added.

    At the same time adds a new comment on the observation with the link to
    redirect

    In order to avoid problems when pulling data from social media, only will
    post to social media when a new contribution is added to any category
    different than 'Tweets'
    """
    from django.contrib.sites.models import Site
    project = instance.project
    socialinteractions_all = SocialInteractionPost.objects.filter(project=project, status='active')

    if instance.category.name != 'Tweets':
        for socialinteraction in socialinteractions_all:
            link = socialinteraction.link
            link = link.replace("$project_id$", str(project.id))
            link = link.replace("$contribution_id$", str(instance.id))

            text_to_post = socialinteraction.text_to_post.replace("$link$", link)

            socialaccount = socialinteraction.socialaccount
            provider = socialaccount.provider
            app = SocialApp.objects.get(provider=provider)

            access_token = SocialToken.objects.get(
                account__id=socialaccount.id,
                account__user=socialaccount.user,
                account__provider=app.provider
            )

            if provider == "twitter":
                text_to_post = text_to_post[:280]

            tweet_id, screen_name = post_to_social_media(
                provider,
                access_token,
                text_to_post,
                app)

            # comment_txt = 'https://twitter.com/{user_name}/status/{tweet_id}'.format(
            #     user_name=screen_name,
            #     tweet_id=tweet_id
            # )
            # Comment.objects.create(
            #     text=comment_txt,
            #     commentto=instance,
            #     creator=socialaccount.user
            # )

        return "posted to social media"

    return "Category name is Tweets"


def post_to_social_media(provider, access_token, text_to_post, app):
    """Check the provider and authenticate.

    Parameters:
    ------------
    provider :  str
        provider of the social account
    access_token: str - SocialToken Object
        access token for the social account and user
    text_to_post: str
        text which will be posted to social media
    app: socialAccount app object

    returns
    --------
    tweet_id : str
        tweet identifier
    screen_aname: str
        screen name user by twitter user

    """
    if provider == 'facebook':
        graph = facebook.GraphAPI(access_token)
        graph.put_wall_post(message=text_to_post)
    if provider == 'twitter':
        consumer_key = app.client_id
        consumer_secret = app.secret
        auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
        access_token_all = access_token
        access_token = access_token_all.token
        access_token_secret = access_token_all.token_secret
        auth.set_access_token(access_token, access_token_secret)
        api = tweepy.API(auth)
        tweet_back = api.update_status(text_to_post)

    return tweet_back.id, tweet_back.author.screen_name



"""Tests for views of social interactions."""

from django.test import TestCase, override_settings
from django.conf import settings
from django.http import HttpRequest, QueryDict
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string
from django.contrib.auth.models import AnonymousUser
from django.contrib.sites.shortcuts import get_current_site
from django.contrib.messages import get_messages
from django.contrib.messages.storage.fallback import FallbackStorage

from allauth.socialaccount.models import SocialAccount
from allauth.socialaccount.providers import registry

from geokey import version
from geokey.core.tests.helpers import render_helpers
from geokey.users.tests.model_factories import UserFactory
from geokey.projects.tests.model_factories import ProjectFactory
from geokey.socialinteractions.base import freq_dic, STATUS

import importlib

from .model_factories import (
    SocialInteractionFactory,
    SocialInteractionPullFactory
)
from ..models import SocialInteractionPost, SocialInteractionPull
from ..views import (
    SocialInteractionList,
    SocialInteractionPostCreate,
    SocialInteractionPostSettings,
    SocialInteractionPostDelete,
    SocialInteractionPost,
    SocialInteractionPullCreate,
    SocialInteractionPullDelete,
    SocialInteractionPullSettings
)


def install_required_apps():
    """Install Twitter and Facebook providers for django-allauth."""
    installed_apps = settings.INSTALLED_APPS

    apps_to_install = [
        'allauth.socialaccount.providers.twitter',
        'allauth.socialaccount.providers.facebook',
    ]

    for app in apps_to_install:
        if app not in installed_apps:
            installed_apps = installed_apps + (app,)
            importlib.import_module(app + '.provider')

    return installed_apps


class SocialInteractionsListTest(TestCase):
    """Test a list of social interactions page."""

    def setUp(self):
        """Set up tests."""
        self.view = SocialInteractionList.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = AnonymousUser()

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser.

        It should redirect to the login page.
        """
        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

    def test_get_with_user(self):
        """
        Accessing the view with normal user.

        It should render the page with an error message.
        """
        user = UserFactory.create()
        project = ProjectFactory.create()

        self.request.user = user
        response = self.view(self.request, project_id=project.id).render()

        rendered = render_to_string(
            'socialinteractions/socialinteraction_list.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_get_with_admin(self):
        """
        Accessing the view with project admin.

        It should render the page.
        """
        project = ProjectFactory.create()
        user = project.creator

        self.request.user = user
        response = self.view(self.request, project_id=project.id).render()

        rendered = render_to_string(
            'socialinteractions/socialinteraction_list.html',
            {
                'project': project,
                'user': user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)


@override_settings(INSTALLED_APPS=install_required_apps())
class SocialInteractionCreateTest(TestCase):
    """Test creating a new social interaction."""

    def setUp(self):
        """Set up tests."""
        self.anonymous_user = AnonymousUser()
        self.regular_user = UserFactory.create()
        self.admin_user = UserFactory.create()

        self.project = ProjectFactory.create(creator=self.admin_user)
        self.socialaccount_1 = SocialAccount.objects.create(
            user=self.regular_user, provider='twitter', uid='1')
        self.socialaccount_2 = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='2')

        self.view = SocialInteractionPostCreate.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = self.anonymous_user

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser.

        It should redirect to the loginpage.
        """
        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

    def test_get_with_user(self):
        """
        Accessing the view with normal user.

        It should render the page with an error message.
        """
        self.request.user = self.regular_user
        response = self.view(self.request, project_id=self.project.id).render()

        rendered = render_to_string(
            'socialinteractions/socialinteraction_post_create.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': self.regular_user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    # def test_get_with_admin(self):
    #     """
    #     Accessing the view with project admin.

    #     It should render the page.
    #     """
    #     self.request.user = self.admin_user
    #     response = self.view(self.request, project_id=self.project.id).render()

    #     rendered = render_to_string(
    #         'socialinteractions/socialinteraction_post_create.html',
    #         {
    #             'project': self.project,
    #             'auth_users': [self.socialaccount_2],
    #             'user': self.admin_user,
    #             'PLATFORM_NAME': get_current_site(self.request).name,
    #             'GEOKEY_VERSION': version.get_version()
    #         }
    #     )
    #     self.assertEqual(response.status_code, 200)
    #     response = render_helpers.remove_csrf(response.content.decode('utf-8'))
    #     self.assertEqual(response, rendered)

    def test_post_with_anonymous(self):
        """
        Updating with AnonymousUser.

        It should redirect to the login page.
        """
        self.request.method = 'POST'
        self.request.POST = {
            'name': 'My social interaction',
            'description': '',
            'socialaccount': self.socialaccount_2.id
        }
        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])
        self.assertEqual(0, SocialInteractionPost.objects.count())

    def test_post_with_user(self):
        """
        Updating with normal user.

        It should render the page with an error message.
        """
        self.request.method = 'POST'
        self.request.POST = {
            'name': 'My social interaction',
            'description': '',
            'socialaccount': self.socialaccount_2.id
        }

        self.request.user = self.regular_user
        response = self.view(self.request, project_id=self.project.id).render()

        rendered = render_to_string(
            'socialinteractions/socialinteraction_post_create.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': self.regular_user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)
        self.assertEqual(0, SocialInteractionPost.objects.count())

    def test_post_with_admin(self):
        """
        Updating with project admin.

        It should create the social interaction and redirect to the social
        interaction page to set the post message.
        """
        self.request.method = 'POST'
        post = QueryDict('socialaccount=%s&text_post=%s&text_link=%s' %
                         (
                             self.socialaccount_2.id,
                             'New contribution created for #project. Check it out here $link$',
                             'http://www.mymapfrontend/$project_id$/$contribution_id$'
                         ))

        self.request.POST = post
        self.request.user = self.admin_user
        response = self.view(self.request, project_id=self.project.id)
        self.assertEqual(1, SocialInteractionPost.objects.count())
        socialinteraction = SocialInteractionPost.objects.first()
        self.assertEqual(socialinteraction.text_to_post,
                         'New contribution created for #project. Check it out here $link$')
        self.assertEqual(socialinteraction.link, 'http://www.mymapfrontend/$project_id$/$contribution_id$')
        self.assertEqual(socialinteraction.project, self.project)
        self.assertEqual(socialinteraction.creator, self.admin_user)
        self.assertEqual(self.socialaccount_2, socialinteraction.socialaccount)

        self.assertEqual(response.status_code, 302)
        self.assertIn(
            '/admin/projects/%s/socialinteractions/' % (
                self.project.id),
            response['location']
        )

    def test_post_on_locked_project_with_admin(self):
        """
        Updating with project admin when the project is locked.

        It should redirect to creating a new social interaction page.
        """
        self.request.method = 'POST'
        post = QueryDict('name=%s&description=''%s&socialaccount=%s' %
                         (
                             'My social interaction',
                             '',
                             self.socialaccount_2.id
                         ))

        self.request.POST = post
        self.project.islocked = True
        self.project.save()

        self.request.user = self.admin_user
        response = self.view(self.request, project_id=self.project.id)

        self.assertEqual(0, SocialInteractionPost.objects.count())
        self.assertEqual(response.status_code, 302)
        self.assertIn(
            '/admin/projects/%s/socialinteractions/post/create' % (self.project.id),
            response['location']
        )

    def test_post_when_social_accounts_do_not_exist_with_admin(self):
        """
        Updating with project admin when the social account is not found.

        It should redirect to creating a new social interaction page.
        """
        self.request.method = 'POST'
        post = QueryDict('name=%s&description=''%s&socialaccount=%s' %
                         (
                             'My social interaction',
                             '',
                             74746464
                         ))

        self.request.POST = post

        self.request.user = self.admin_user
        response = self.view(self.request, project_id=self.project.id)

        self.assertEqual(0, SocialInteractionPost.objects.count())
        self.assertEqual(response.status_code, 302)
        self.assertIn(
            '/admin/projects/%s/socialinteractions/post/create' % (self.project.id),
            response['location'])


@override_settings(INSTALLED_APPS=install_required_apps())
class SocialInteractionSettingsTest(TestCase):
    """Test social interaction settings page."""

    def setUp(self):
        """Set up tests."""
        self.anonymous_user = AnonymousUser()
        self.regular_user = UserFactory.create()
        self.admin_user = UserFactory.create()

        self.project = ProjectFactory.create(creator=self.admin_user)

        self.socialaccount_2 = SocialAccount.objects.create(
            user=self.admin_user, provider='facebook', uid='2')
        self.socialaccount_1 = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='1')
        self.socialaccount_3 = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='3')
        self.socialinteraction = SocialInteractionFactory.create(
            socialaccount=self.socialaccount_1,
            project=self.project,
            creator=self.admin_user
        )
        self.view = SocialInteractionPostSettings.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = self.anonymous_user

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser.

        It should redirect to the login page.
        """
        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])
        self.assertEqual(SocialInteractionPost.objects.count(), 1)

    def test_get_with_user(self):
        """
        Accessing the view with normal user.

        It should render the page with an error message.
        """
        self.request.user = self.regular_user
        response = self.view(
            self.request,
            project_id=self.project.id,
            socialinteraction_id=self.socialinteraction.id
        ).render()

        rendered = render_to_string(
            'socialinteractions/socialinteraction_post_settings.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': self.regular_user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    # def test_get_with_admin(self):
    #     """
    #     Accessing the view with project admin.

    #     It should render the page.
    #     """
    #     self.socialinteraction.creator = self.admin_user
    #     self.socialinteraction.project = self.project
    #     self.request.user = self.socialinteraction.creator
    #     self.socialinteraction.save()
    #     response = self.view(
    #         self.request,
    #         project_id=self.project.id,
    #         socialinteraction_id=self.socialinteraction.id
    #     ).render()

    #     socialaccounts_log = SocialAccount.objects.filter(
    #         user=self.admin_user,
    #         provider__in=[id for id, name in registry.as_choices()
    #                       if id in ['twitter', 'facebook']]
    #     )
    #     rendered = render_to_string(
    #         'socialinteractions/socialinteraction_post_settings.html',
    #         {
    #             'project': self.socialinteraction.project,
    #             'socialinteraction': self.socialinteraction,
    #             'auth_users': socialaccounts_log,
    #             'user': self.admin_user,
    #             'PLATFORM_NAME': get_current_site(self.request).name,
    #             'GEOKEY_VERSION': version.get_version()
    #         }
    #     )
    #     self.assertEqual(response.status_code, 200)
    #     response = render_helpers.remove_csrf(response.content.decode('utf-8'))
    #     self.assertEqual(response, rendered)

    def test_post_with_anonymous(self):
        """
        Updating with AnonymousUser.

        It should redirect to the login page.
        """
        self.request.method = 'POST'
        reference = SocialInteractionPost.objects.get(pk=self.socialinteraction.id)

        post = QueryDict('socialaccount=%s&text_post=%s&text_link=%s' %
                         (
                             self.socialaccount_3.id,
                             'New contribution created for #project. Check it out here $link$',
                             'http://www.mymapfrontend/$project_id$/$contribution_id$'
                         ))

        self.request.POST = post
        response = self.view(
            self.request,
            project_id=self.socialinteraction.project.id,
            socialinteraction_id=self.socialinteraction.id)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])
        reference = SocialInteractionPost.objects.get(pk=self.socialinteraction.id)
        self.assertNotEqual(reference.text_to_post, post['text_post'])
        self.assertNotEqual(reference.link, post['text_link'])
        socialaccount = reference.socialaccount
        self.assertNotEqual(self.socialaccount_3, socialaccount)

    def test_post_with_user(self):
        """
        Updating with normal user.

        It should render the page with an error message.
        """
        self.request.method = 'POST'
        post = QueryDict('socialaccount=%s&text_post=%s&text_link=%s' %
                         (
                             self.socialaccount_3.id,
                             'New contribution created for #project. Check it out here $link$',
                             'http://www.mymapfrontend/$project_id$/$contribution_id$'
                         ))

        self.request.POST = post

        self.request.user = self.regular_user
        response = self.view(
            self.request,
            project_id=self.project.id,
            socialinteraction_id=self.socialinteraction.id
        ).render()

        rendered = render_to_string(
            'socialinteractions/socialinteraction_post_settings.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': self.regular_user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

        reference = SocialInteractionPost.objects.get(pk=self.socialinteraction.id)
        self.assertNotEqual(reference.text_to_post,
                            'New contribution created for #project. Check it out here $link$')
        self.assertNotEqual(reference.link, 'http://www.mymapfrontend/$project_id$/$contribution_id$')
        socialaccount = reference.socialaccount
        self.assertNotEqual(self.socialaccount_3, socialaccount)


class SocialInteractionDeleteTest(TestCase):
    """Test social interaction delete view."""

    def setUp(self):
        """Set up test."""
        self.anonymous_user = AnonymousUser()
        self.regular_user = UserFactory.create()
        self.admin_user = UserFactory.create()

        self.project = ProjectFactory.create(creator=self.admin_user)

        self.socialaccount_2 = SocialAccount.objects.create(
            user=self.admin_user, provider='facebook', uid='2')
        self.socialaccount_1 = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='1')
        self.socialaccount_3 = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='3')
        self.socialinteraction = SocialInteractionFactory.create(
            socialaccount=self.socialaccount_1,
            project=self.project,
            creator=self.admin_user
        )
        self.view = SocialInteractionPostDelete.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = self.anonymous_user

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser.

        It should redirect to the login page.
        """
        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])
        self.assertEqual(SocialInteractionPost.objects.count(), 1)

    def test_get_with_user(self):
        """
        Accessing the view with normal user.

        It should render the page with an error message.
        """
        self.request.user = self.regular_user
        response = self.view(
            self.request,
            project_id=self.project.id,
            socialinteraction_id=self.socialinteraction.id
        ).render()

        rendered = render_to_string(
            'base.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': self.regular_user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_get_with_admin(self):
        """
        Accessing the view with project admin.

        It should render the page.
        """
        self.socialinteraction.project = self.project
        self.socialinteraction.creator = self.admin_user
        self.socialinteraction.save()

        self.request.user = self.admin_user
        response = self.view(
            self.request,
            project_id=self.project.id,
            socialinteraction_id=self.socialinteraction.id
        )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(SocialInteractionPost.objects.count(), 0)

    def test_delete_with_admin_when_project_is_locked(self):
        """
        Accessing the view with project admin when project is locked.

        It should render the page.
        """
        self.project.islocked = True
        self.project.save()
        self.socialinteraction.project = self.project
        self.socialinteraction.creator = self.admin_user
        self.socialinteraction.save()

        self.request.user = self.admin_user
        response = self.view(
            self.request,
            project_id=self.project.id,
            socialinteraction_id=self.socialinteraction.id
        )

        self.assertEqual(response.status_code, 302)
        self.assertIn(
            reverse(
                'admin:socialinteraction_list',
                args=(self.project.id,)
            ),
            response['location']
        )
        self.assertEqual(SocialInteractionPost.objects.count(), 1)

    def test_delete_with_admin_when_project_does_not_exit(self):
        """
        Accessing the view with project admin when project does not exist.

        It should render the page with an error message.
        """
        self.socialinteraction.project = self.project
        self.socialinteraction.creator = self.admin_user
        self.socialinteraction.save()
        self.request.user = self.admin_user
        response = self.view(
            self.request,
            project_id=634842156456,
            socialinteraction_id=self.socialinteraction.id
        ).render()

        rendered = render_to_string(
            'base.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': self.admin_user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)
        self.assertEqual(SocialInteractionPost.objects.count(), 1)


@override_settings(INSTALLED_APPS=install_required_apps())
class SocialInteractionPullCreateTest(TestCase):
    """Test creating a new social interaction pull."""

    def setUp(self):
        """Set up tests."""
        self.anonymous_user = AnonymousUser()
        self.regular_user = UserFactory.create()
        self.admin_user = UserFactory.create()

        self.project = ProjectFactory.create(creator=self.admin_user)
        self.socialaccount_1 = SocialAccount.objects.create(
            user=self.regular_user, provider='twitter', uid='1')
        self.socialaccount_2 = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='2')

        self.view = SocialInteractionPullCreate.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = self.anonymous_user

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser.

        It should redirect to the loginpage.
        """
        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

    def test_get_with_user(self):
        """
        Accessing the view with normal user.

        It should render the page with an error message.
        """
        self.request.user = self.regular_user
        response = self.view(self.request, project_id=self.project.id).render()

        rendered = render_to_string(
            'socialinteractions/socialinteraction_pull_create.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': self.regular_user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

        # def test_get_with_admin(self):
        #     """
        #     Accessing the view with project admin.

        #     It should render the page.
        #     """
        #     self.request.user = self.admin_user
        #     response = self.view(self.request, project_id=self.project.id).render()

        #     rendered = render_to_string(
        #         'socialinteractions/socialinteraction_pull_create.html',
        #         {
        #             'project': self.project,
        #             'auth_users': [
        #                 self.socialaccount_2,
        #                 self.socialaccount_1
        #             ],
        #             'user': self.admin_user,
        #             'PLATFORM_NAME': get_current_site(self.request).name,
        #             'GEOKEY_VERSION': version.get_version()
        #         }
        #     )
        #     self.assertEqual(response.status_code, 200)
        #     response = render_helpers.remove_csrf(response.content.decode('utf-8'))
        #     # self.assertEqual(response, rendered)


class SocialInteractionPullDeleteTest(TestCase):
    """Test social interaction pull delete view."""

    def setUp(self):
        """Set up test."""
        self.anonymous_user = AnonymousUser()
        self.regular_user = UserFactory.create()
        self.admin_user = UserFactory.create()

        self.project = ProjectFactory.create(creator=self.admin_user)

        self.socialaccount_2 = SocialAccount.objects.create(
            user=self.admin_user, provider='facebook', uid='2')
        self.socialaccount_1 = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='1')
        self.socialaccount_3 = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='3')
        self.si_pull = SocialInteractionPullFactory.create(
            socialaccount=self.socialaccount_1,
            project=self.project,
            creator=self.admin_user
        )

        self.view = SocialInteractionPullDelete.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = self.anonymous_user

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser.

        It should redirect to the login page.
        """
        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])
        self.assertEqual(SocialInteractionPull.objects.count(), 1)

    def test_get_with_user(self):
        """
        Accessing the view with normal user.

        It should render the page with an error message.
        """
        self.request.user = self.regular_user
        response = self.view(
            self.request,
            project_id=self.project.id,
            socialinteractionpull_id=self.si_pull.id
        ).render()

        rendered = render_to_string(
            'base.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': self.regular_user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    def test_get_with_admin(self):
        """
        Accessing the view with project admin.

        It should render the page.
        """
        self.si_pull.project = self.project
        self.si_pull.creator = self.admin_user
        self.si_pull.save()

        self.request.user = self.admin_user
        response = self.view(
            self.request,
            project_id=self.project.id,
            socialinteractionpull_id=self.si_pull.id
        )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(SocialInteractionPull.objects.count(), 0)

    def test_delete_with_admin_when_project_is_locked(self):
        """
        Accessing the view with project admin when project is locked.
        It should render the page.
        """
        self.project.islocked = True
        self.project.save()
        self.si_pull.project = self.project
        self.si_pull.creator = self.admin_user
        self.si_pull.save()
        self.request.user = self.admin_user
        response = self.view(
            self.request,
            project_id=self.project.id,
            socialinteractionpull_id=self.si_pull.id
        )
        self.assertEqual(response.status_code, 302)
        self.assertIn(
            reverse(
                'admin:socialinteraction_list',
                args=(self.project.id,)
            ),
            response['location']
        )
        self.assertEqual(SocialInteractionPull.objects.count(), 1)

    def test_delete_with_admin_when_project_does_not_exit(self):
        """
        Accessing the view with project admin when project does not exist.

        It should render the page with an error message.
        """
        self.si_pull.project = self.project
        self.si_pull.creator = self.admin_user
        self.si_pull.save()
        self.request.user = self.admin_user
        response = self.view(
            self.request,
            project_id=634842156456,
            socialinteractionpull_id=self.si_pull.id
        ).render()

        rendered = render_to_string(
            'base.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': self.admin_user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)
        self.assertEqual(SocialInteractionPull.objects.count(), 1)


@override_settings(INSTALLED_APPS=install_required_apps())
class SocialInteractionPullSettingsTest(TestCase):
    """Test social interaction pull settings page."""

    def setUp(self):
        """Set up tests."""
        self.anonymous_user = AnonymousUser()
        self.regular_user = UserFactory.create()
        self.admin_user = UserFactory.create()

        self.project = ProjectFactory.create(creator=self.admin_user)

        self.socialaccount_2 = SocialAccount.objects.create(
            user=self.admin_user, provider='facebook', uid='2')
        self.socialaccount_1 = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='1')
        self.socialaccount_3 = SocialAccount.objects.create(
            user=self.admin_user, provider='twitter', uid='3')
        self.si_pull = SocialInteractionPullFactory.create(
            socialaccount=self.socialaccount_1,
            project=self.project,
            creator=self.admin_user
        )
        self.view = SocialInteractionPullSettings.as_view()
        self.request = HttpRequest()
        self.request.method = 'GET'
        self.request.user = self.anonymous_user

        self.freq = freq_dic.keys()
        refund_dict = {value: key for key, value in STATUS}
        self.status_types = refund_dict.keys()

        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_get_with_anonymous(self):
        """
        Accessing the view with AnonymousUser.

        It should redirect to the login page.
        """
        response = self.view(self.request)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])
        self.assertEqual(SocialInteractionPull.objects.count(), 1)

    def test_get_with_user(self):
        """
        Accessing the view with normal user.

        It should render the page with an error message.
        """
        self.request.user = self.regular_user
        response = self.view(
            self.request,
            project_id=self.project.id,
            socialinteractionpull_id=self.si_pull.id
        ).render()

        rendered = render_to_string(
            'socialinteractions/socialinteraction_pull.html',
            {
                'error_description': 'Project matching query does not exist.',
                'error': 'Not found.',
                'user': self.regular_user,
                'PLATFORM_NAME': get_current_site(self.request).name,
                'GEOKEY_VERSION': version.get_version()
            }
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content.decode('utf-8'), rendered)

    # def test_get_with_admin(self):
    #     """
    #     Accessing the view with project admin.

    #     It should render the page.
    #     """
    #     socialaccounts_log = SocialAccount.objects.filter(
    #         user=self.admin_user,
    #         provider__in=[id for id, name in registry.as_choices()
    #                       if id in ['twitter', 'facebook']]
    #     )
    #     self.si_pull.creator = self.admin_user
    #     self.si_pull.project = self.project
    #     self.request.user = self.si_pull.creator
    #     self.si_pull.save()
    #     response = self.view(
    #         self.request,
    #         project_id=self.project.id,
    #         socialinteractionpull_id=self.si_pull.id
    #     ).render()

    #     socialaccounts_log = SocialAccount.objects.filter(
    #         user=self.admin_user,
    #         provider__in=[id for id, name in registry.as_choices()
    #                       if id in ['twitter', 'facebook']]
    #     )

    #     rendered = render_to_string(
    #         'socialinteractions/socialinteraction_pull.html',
    #         {
    #             'project': self.si_pull.project,
    #             'auth_users': socialaccounts_log,
    #             'socialinteraction_pull': self.si_pull,
    #             'user': self.admin_user,
    #             'status_types': self.status_types,
    #             'freq': self.freq,
    #             'PLATFORM_NAME': get_current_site(self.request).name,
    #             'GEOKEY_VERSION': version.get_version()
    #         }
    #     )
    #     self.assertEqual(response.status_code, 200)
    #     response = render_helpers.remove_csrf(response.content.decode('utf-8'))
    #     self.assertEqual(response, rendered)

    def test_post_with_anonymous(self):
        """
        Updating with AnonymousUser.

        It should redirect to the login page.
        """
        self.request.method = 'POST'
        post = QueryDict('frequency=%s&text_pull=%s&status_type=%s&socialaccount=%s' % (
            'fortnigthly',
            '#hastag',
            'inactive',
            self.socialaccount_3.id,
        ))

        self.request.POST = post
        response = self.view(
            self.request,
            project_id=self.si_pull.project.id,
            socialinteractionpull_id=self.si_pull.id)

        self.assertEqual(response.status_code, 302)
        self.assertIn('/admin/account/login/', response['location'])

        reference = SocialInteractionPull.objects.get(id=self.si_pull.id)
        self.assertNotEqual(reference.frequency, 'fortnigthly')
        self.assertNotEqual(reference.text_to_pull, '#hastag')
        self.assertNotEqual(reference.status, 'inactive')
        socialaccount = reference.socialaccount
        self.assertNotEqual(self.socialaccount_3, socialaccount)

        # def test_post_with_user(self):
        #     """
        #     Updating with normal user.
        #
        #     It should render the page with an error message.
        #     """
        #     self.request.method = 'POST'
        #     post = QueryDict('frequency=%s&text_pull=%s&status_type=%s&socialaccount=%s' % (
        #         'fortnigthly',
        #         '#hastag',
        #         'inactive',
        #         self.socialaccount_3.id,
        #     ))
        #     self.request.POST = post
        #
        #     self.request.user = self.regular_user
        #     response = self.view(
        #         self.request,
        #         project_id=self.si_pull.project.id,
        #         socialinteractionpull_id=self.si_pull.id
        #     ).render()
        #
        #     rendered = render_to_string(
        #         'socialinteractions/socialinteraction_pull.html',
        #         {
        #             'error_description': 'Project matching query does not exist.',
        #             'error': 'Not found.',
        #             'user': self.regular_user,
        #             'PLATFORM_NAME': get_current_site(self.request).name,
        #             'GEOKEY_VERSION': version.get_version()
        #         }
        #     )
        #     self.assertEqual(response.status_code, 200)
        #     self.assertEqual(response.content.decode('utf-8'), rendered)
        #
        #     reference = SocialInteractionPull.objects.get(id=self.si_pull.id)
        #     self.assertEqual(str(reference.frequency), 'fortnigthly')
        #     self.assertEqual(str(reference.status), 'inactive')
        #     self.assertEqual(str(reference.text_to_pull), '#hastag')
        #     socialaccount = reference.socialaccount
        #     self.assertEqual(self.socialaccount_3, socialaccount)

        # def test_post_with_admin(self):
        #     """
        #     Updating with admin user.

        #     It should render the page with an error message.
        #     """
        #     self.request.method = 'POST'
        #     post = QueryDict('frequency=%s&text_pull=%s&status_type=%s&socialaccount=%s' % (
        #         'hourly',
        #         '#hastag',
        #         'inactive',
        #         self.socialaccount_3.id,
        #     ))
        #     self.request.POST = post

        #     socialaccounts_log = SocialAccount.objects.filter(
        #         user=self.admin_user,
        #         provider__in=[id for id, name in registry.as_choices()
        #                       if id in ['twitter', 'facebook']]
        #     )

        #     self.request.user = self.admin_user
        #     response = self.view(
        #         self.request,
        #         project_id=self.si_pull.project.id,
        #         socialinteractionpull_id=self.si_pull.id
        #     ).render()

        #     rendered = render_to_string(
        #         'socialinteractions/socialinteraction_pull.html',
        #         {
        #             'project': self.si_pull.project,
        #             'auth_users': socialaccounts_log,
        #             'socialinteraction_pull': self.si_pull,
        #             'user': self.admin_user,
        #             'status_types': self.status_types,
        #             'freq': self.freq,
        #             'PLATFORM_NAME': get_current_site(self.request).name,
        #             'GEOKEY_VERSION': version.get_version()
        #         }
        #     )
        #     self.assertEqual(response.status_code, 200)
        #     self.assertEqual(response.content.decode('utf-8'), rendered)

        #     reference = SocialInteractionPull.objects.get(id=self.si_pull.id)
        #     self.assertEqual(str(reference.frequency), 'hourly')
        #     self.assertEqual(str(reference.status), 'inactive')
        #     self.assertEqual(str(reference.text_to_pull), '#hastag')
        #     socialaccount = reference.socialaccount
        #     self.assertEqual(self.socialaccount_3, socialaccount)



"""Document helpers of contributions."""

from os.path import dirname, normpath, abspath, join

from django.core.files import File
from django.core.files.base import ContentFile


def get_pdf_document(file_name='document_1.pdf'):
    pdf_file = File(open(
        normpath(join(
            dirname(dirname(abspath(__file__))),
            'files/document_1.pdf'
        )),
        'rb'
    ))

    the_file = ContentFile(pdf_file.read(), file_name)
    the_file.content_type = 'application/pdf'

    return the_file


def get_doc_document(file_name='document_2.doc'):
    doc_file = File(open(
        normpath(join(
            dirname(dirname(abspath(__file__))),
            'files/document_2.doc'
        )),
        'rb'
    ))

    the_file = ContentFile(doc_file.read(), file_name)
    the_file.content_type = 'application/msword'

    return the_file



"""Views for social interactions."""

from django.views.generic import TemplateView
from django.core.urlresolvers import reverse
from django.utils.html import strip_tags
from django.utils.safestring import mark_safe
from django.shortcuts import redirect
from django.contrib import messages

from braces.views import LoginRequiredMixin
from allauth.socialaccount.models import SocialAccount, SocialToken, SocialApp

from geokey.core.decorators import handle_exceptions_for_admin
from geokey.projects.models import Project
from geokey.projects.views import ProjectContext

from .models import SocialInteractionPost, SocialInteractionPull
from .base import STATUS, FREQUENCY


class SocialInteractionList(LoginRequiredMixin, ProjectContext, TemplateView):
    """
    Displays the list of social interactions in the project.
    """
    template_name = 'socialinteractions/socialinteraction_list.html'


class SocialInteractionPostCreate(LoginRequiredMixin, ProjectContext, TemplateView):
    """
    Provides the form to create a new social interaction.
    """
    template_name = 'socialinteractions/socialinteraction_post_create.html'

    def get_context_data(self, *args, **kwargs):

        context = super(SocialInteractionPostCreate, self).get_context_data(
            *args,
            **kwargs
        )

        auth_users = SocialAccount.objects.filter(
            user=self.request.user,
            provider__in=['twitter', 'facebook'])

        context["auth_users"] = auth_users
        return context

    def post(self, request, project_id):
        """
        Creates the social interaction based on the data entered by the user.

        Parameters
        ----------
        request : django.http.HttpRequest
            Object representing the request
        project_id : int
            Identifies the project in the database

        Returns
        -------
        django.http.HttpResponseRedirect
            Redirects to social interaction create if social interaction is
            created, social interaction list if project is locked or it does
            not have any categories
        django.http.HttpResponse
            Rendered template, if project does not exist
        """

        data = request.POST
        context = self.get_context_data(project_id)
        project = context.get('project')

        if project:
            cannot_create = 'New social interactions cannot be created.'

            if project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. %s' % cannot_create
                )
                return redirect(
                    'admin:socialinteraction_post_create',
                    project_id=project_id
                )

            try:
                socialaccount = SocialAccount.objects.get(
                    pk=data.get('socialaccount'))
                text_to_post = data.get('text_post')
                link = data.get('text_link')
            except SocialAccount.DoesNotExist:
                messages.error(
                    self.request,
                    'The social account is not found. %s' % cannot_create
                )
                return redirect(
                    'admin:socialinteraction_post_create',
                    project_id=project_id
                )

            socialinteraction = SocialInteractionPost.objects.create(
                creator=request.user,
                project=project,
                socialaccount=socialaccount,
                text_to_post=text_to_post,
                link=link,
            )

            add_another_url = reverse(
                'admin:socialinteraction_post_create',
                kwargs={
                    'project_id': project_id
                }
            )

            messages.success(
                self.request,
                mark_safe(
                    'The social interaction has been created.<a href="%s"> Add another social interaction.</a>' % add_another_url)
            )
            return redirect(
                'admin:socialinteraction_list',
                project_id=project_id,
            )

        else:
            return self.render_to_response(context)


class SocialInteractionPostContext(object):
    """
    Provides the context to render templates. The context contains
    a social interaction instance based on project_id and socialinteraction_id.
    """

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, socialinteraction_id, *args, **kwargs):
        """
        Returns the context containing the project and social interaction
        instances.

        Parameters
        ----------
        project_id : int
            Identifies the project in the database
        socialinteraction_id : int
            Identifies the social interaction in the database

        Returns
        -------
        dict
            Context
        """

        project = Project.objects.as_admin(self.request.user, project_id)

        try:
            socialinteraction = project.socialinteractions_post.get(
                id=socialinteraction_id)

        except:
            messages.error(
                self.request, 'The social interaction was not found.'
            )
            return redirect(
                'socialinteractions/socialinteraction_post_settings.html',
                project_id=project_id,
                socialinteraction_id=socialinteraction_id,
            )

        try:
            socialaccount = SocialAccount.objects.get(id=socialinteraction.socialaccount_id)
        except:
            messages.error(
                self.request, 'The social account was not found'
            )
            return redirect(
                'socialinteractions/socialinteraction_post_settings.html',
                project_id=project_id,
                socialinteraction_id=socialinteraction_id,
            )

        if socialinteraction and socialaccount:
            return super(SocialInteractionPostContext, self).get_context_data(
                project=project,
                socialinteraction=socialinteraction,
                socialaccount=socialaccount,
            )


class SocialInteractionPostDelete(LoginRequiredMixin, SocialInteractionPostContext,
                                  TemplateView):
    """
    Deletes the social interactions.
    """
    template_name = 'base.html'

    def get(self, request, project_id, socialinteraction_id):
        """
        Deletes the social interaction.

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request
        project_id : int
            Identifies the project in the database
        socialinteraction_id : int
            Identifies the social interaction in the database

        Returns
        -------
        django.http.HttpResponseRedirect
            Redirects to social interaction list if social interaction is
            deleted, social interaction settings if project is locked, if social
            interaction does not exists redirect to base.html and show error
        django.http.HttpResponse
            Rendered template, if project or social interaction does not exist
        """

        try:
            context = self.get_context_data(project_id, socialinteraction_id)
            socialinteraction = context.get('socialinteraction')

        except:
            messages.error(
                self.request, 'The social account is not found.'
            )
            return redirect(
                'base.html',
                project_id=project_id,
                socialinteraction_id=socialinteraction_id
            )

        if socialinteraction:
            if socialinteraction.project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. Social interaction cannot be deleted.'
                )
                return redirect(
                    'admin:socialinteraction_post_settings',
                    project_id=project_id,
                    socialinteraction_id=socialinteraction_id
                )
            else:
                socialinteraction.delete()
                messages.success(self.request, 'The social interaction has been'
                                               ' deleted.')
                return redirect('admin:socialinteraction_list',
                                project_id=project_id)

        return self.render_to_response(context)


class SocialInteractionPostSettings(LoginRequiredMixin, SocialInteractionPostContext,
                                    TemplateView):
    """
    Provides the form to update the social interaction settings.
    """
    template_name = 'socialinteractions/socialinteraction_post_settings.html'

    def get_context_data(self, project_id, *args, **kwargs):
        """
        Return the context to render the view.

        Add Twitter and Facebook social accounts of a user to the context.

        Parameters
        ----------
        project_id : int
            Identifies the project in the database.

        Returns
        -------
        dict
            Context.
        """
        context = super(SocialInteractionPostSettings, self).get_context_data(
            project_id,
            *args,
            **kwargs
        )

        auth_users = SocialAccount.objects.filter(
            user=self.request.user,
            provider__in=['twitter', 'facebook'])

        context["auth_users"] = auth_users
        context['status_types'] = {value: key for key, value in STATUS}.keys()

        return context

    def post(self, request, project_id, socialinteraction_id):
        """
        Updates the social interaction based on the data entered by the user.

        Parameter
        ---------
        request : django.http.HttpRequest
            Object representing the request
        project_id : int
            Identifies the project in the database
        socialinteraction_id : int
            Identifies the scoial interaction in the database

        Returns
        -------
        django.http.HttpResponse
            Rendered template when social interactions updated
        django.http.HttpResponse
            Rendered template, if project or social interaction does not exist
        """

        data = request.POST
        try:
            context = self.get_context_data(project_id, socialinteraction_id)
            socialinteraction = context.get('socialinteraction')
        except:
            messages.error(
                self.request, 'The social account is not found.'
            )
            return redirect(
                'socialinteractions/socialinteraction_post_settings.html',
                project_id=project_id,
                socialinteraction_id=socialinteraction_id
            )

        if socialinteraction:
            if socialinteraction.project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. Social interaction cannot be deleted.'
                )
                return redirect(
                    'admin:socialinteraction_post_settings',
                    project_id=project_id,
                    socialinteraction_id=socialinteraction_id
                )
            else:

                socialinteraction.text_to_post = data.get('text_post')
                socialinteraction.link = data.get('text_link')
                socialinteraction.socialaccount = SocialAccount.objects.get(
                    id=data.get('socialaccount'))
                socialinteraction.status = data.get('status_type')
                socialinteraction.save()

                messages.success(self.request, 'The social interaction has been updated.')

        return self.render_to_response(context)


# class SocialInteractionPost(LoginRequiredMixin, SocialInteractionPostContext,
#                             TemplateView):
#     """Provide the form to update the social interaction settings."""
#
#     template_name = 'socialinteractions/socialinteraction_post.html'
#
#     def get_context_data(self, project_id, *args, **kwargs):
#         """
#         Return the context to render the view.
#
#         Add Twitter and Facebook social accounts of a user to the context.
#
#         Parameters
#         ----------
#         project_id : int
#             Identifies the project in the database.
#
#         Returns
#         -------
#         dict
#             Context.
#         """
#
#         return super(SocialInteractionPost, self).get_context_data(
#             project_id,
#             *args,
#             **kwargs
#         )
#
#     def post(self, request, project_id, socialinteraction_id):
#         """
#         Creates social post base on the data entered by the user.
#
#         Parameters
#         ---------
#         request : django.http.HttpRequest
#             Object representing the request.
#         project_id : intyes
#             Identifies the project in the database.
#         socialinteraction_id : int
#             Identifies the social interaction in the database.
#
#         Returns
#         -------
#         django.http.HttpResponse
#             Rendered template when social interactions updated.
#         django.http.HttpResponse
#             Rendered template, if project or social interaction does not exist.
#         """
#         data = request.POST
#         context = self.get_context_data(project_id, socialinteraction_id)
#         socialinteraction = context.get('socialinteraction')
#         socialinteraction.text_to_post = data.get('text_post')
#         socialinteraction.link = data.get('text_link')
#         socialinteraction.save()
#
#         return self.render_to_response(context)


class SocialInteractionPullCreate(LoginRequiredMixin, ProjectContext,
                                  TemplateView):
    """Provide the form to update the social interaction settings."""

    template_name = 'socialinteractions/socialinteraction_pull_create.html'

    def get_context_data(self, *args, **kwargs):

        context = super(SocialInteractionPullCreate, self).get_context_data(
            *args,
            **kwargs
        )

        auth_users = SocialAccount.objects.filter(
            user=self.request.user,
            provider__in=['twitter', 'facebook'])

        context["auth_users"] = auth_users
        context["frequencies"] = [x for x, _ in FREQUENCY]
        return context

    def post(self, request, project_id):
        """
        Creates social post base on the data entered by the user.

        Parameters
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : intyes
            Identifies the project in the database.
        socialinteraction_id : int
            Identifies the social interaction in the database.

        Returns
        -------
        django.http.HttpResponse
            Rendered template when social interactions updated.
        django.http.HttpResponse
            Rendered template, if project or social interaction does not exist.
        """
        data = request.POST
        context = self.get_context_data(project_id)
        project = context.get('project')

        if project:
            cannot_create = 'New social interactions cannot be created.'

            if project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. %s' % cannot_create
                )
                return redirect(
                    'admin:socialinteraction_post_create',
                    project_id=project_id
                )

            try:
                socialaccount = SocialAccount.objects.get(
                    pk=data.get('socialaccount'))
            except SocialAccount.DoesNotExist:
                messages.error(
                    self.request,
                    'The social account is not found. %s' % cannot_create
                )
                return redirect(
                    'admin:socialinteraction_post_create',
                    project_id=project_id
                )

            SocialInteractionPull.objects.create(
                text_to_pull=strip_tags(data.get('text_pull')),
                creator=request.user,
                project=project,
                socialaccount=socialaccount,
                frequency=strip_tags(data.get('frequency')),
            )

            add_another_url = reverse(
                'admin:socialinteraction_pull_create',
                kwargs={
                    'project_id': project_id
                }
            )

            messages.success(
                self.request,
                mark_safe(
                    'The social interaction has been created.<a href="%s"> Add pull from social media task.</a>' % add_another_url)
            )

            return redirect(
                'admin:socialinteraction_list',
                project_id=project_id,
            )
        else:
            return self.render_to_response(context)


class SocialInteractionPullContext(object):
    """
    Provides the context to render templates. The context contains
    a social interaction instance based on project_id and socialinteraction_id.
    """

    @handle_exceptions_for_admin
    def get_context_data(self, project_id, socialinteractionpull_id, *args, **kwargs):
        """
        Returns the context containing the project and social interaction
        instances.

        Parameters
        ----------
        project_id : int
            Identifies the project in the database
        socialinteraction_id : int
            Identifies the social interaction in the database

        Returns
        -------
        dict
            Context
        """

        project = Project.objects.as_admin(self.request.user, project_id)

        try:
            socialinteraction_pull = project.socialinteractions_pull.get(
                pk=socialinteractionpull_id)

        except:
            messages.error(
                self.request, 'The social interaction pull is not found.'
            )
            return redirect(
                'socialinteractions/socialinteraction_list.html',
                project_id=project_id
            )

        if socialinteraction_pull:
            return super(SocialInteractionPullContext, self).get_context_data(
                project=project,
                socialinteraction_pull=socialinteraction_pull,
            )


class SocialInteractionPullSettings(LoginRequiredMixin, SocialInteractionPullContext,
                                    TemplateView):
    """Provide the form to update the social interaction settings."""

    template_name = 'socialinteractions/socialinteraction_pull.html'

    def get_context_data(self, project_id, *args, **kwargs):

        context = super(SocialInteractionPullSettings, self).get_context_data(
            project_id,
            *args,
            **kwargs
        )

        auth_users = SocialAccount.objects.filter(
            user=self.request.user,
            provider__in=['twitter', 'facebook'])

        context["auth_users"] = auth_users
        context['status_types'] = {value: key for key, value in STATUS}.keys()
        context["freq"] = [x for x, _ in FREQUENCY]

        return context

    def post(self, request, project_id, socialinteractionpull_id):
        """
        Get the data from the specified social media

        Parameters
        ---------
        request : django.http.HttpRequest
            Object representing the request.
        project_id : intyes
            Identifies the project in the database.

        Returns
        -------
        django.http.HttpResponse
            Rendered template when social interactions updated.
        django.http.HttpResponse
            Rendered template, if project or social interaction does not exist.
        """
        data = request.POST
        try:
            context = self.get_context_data(
                project_id,
                socialinteractionpull_id
            )
            si_pull = context['socialinteraction_pull']
        except:
            messages.error(
                self.request, 'The social account is not found.'
            )
            return redirect(
                'socialinteractions/socialinteraction_pull.html',
                project_id=project_id,
                socialinteractionpull_id=socialinteractionpull_id
            )
        if si_pull:
            if si_pull.project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. Social interaction cannot be edited.'
                )
                return redirect(
                    'admin:socialinteraction_pull',
                    project_id=project_id,
                    socialinteractionpull_id=socialinteractionpull_id
                )
            else:
                text_pull = data.get("text_pull")
                frequency = data.get('frequency')
                socialaccount_id = data.get('socialaccount')
                socialaccount = SocialAccount.objects.get(id=socialaccount_id)

                status = data.get('status_type')

                if text_pull != si_pull.text_to_pull:
                    si_pull.text_to_pull = text_pull
                if si_pull.frequency != frequency:
                    si_pull.frequency = frequency
                if si_pull.socialaccount != socialaccount:
                    si_pull.socialaccount = socialaccount
                if si_pull.status != status:
                    si_pull.status = status
                si_pull.save()

                messages.success(self.request, 'The social interaction has been updated.')

        return self.render_to_response(context)


class SocialInteractionPullDelete(LoginRequiredMixin, SocialInteractionPullContext,
                                  TemplateView):
    """
    Deletes the social interactions.
    """
    template_name = 'base.html'

    def get(self, request, project_id, socialinteractionpull_id):
        try:
            context = self.get_context_data(project_id, socialinteractionpull_id)
            socialinteraction_pull = context.get('socialinteraction_pull')
        except:
            messages.error(
                self.request, 'The social account is not found.'
            )
            return redirect(
                'base.html',
                project_id=project_id,
                socialinteractionpull_id=socialinteraction_pull.id
            )

        if socialinteraction_pull:
            if socialinteraction_pull.project.islocked:
                messages.error(
                    self.request,
                    'The project is locked. Social pull cannot be deleted.'
                )
                return redirect(
                    'admin:socialinteraction_pull_settings',
                    project_id=project_id,
                    socialinteractionpull_id=socialinteraction_pull.id
                )
            else:
                socialinteraction_pull.delete()
                messages.success(self.request, 'The social interaction has been'
                                               ' deleted.')
                return redirect('admin:socialinteraction_list',
                                project_id=project_id)

        return self.render_to_response(context)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contributions', '0018_historicalcomment'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentFile',
            fields=[
                ('mediafile_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='contributions.MediaFile')),
                ('document', models.FileField(upload_to=b'user-uploads/documents')),
                ('thumbnail', models.ImageField(null=True, upload_to=b'user-uploads/documents')),
            ],
            options={
                'ordering': ['id'],
            },
            bases=('contributions.mediafile',),
        ),
        migrations.AlterModelOptions(
            name='mediafile',
            options={},
        ),
    ]



"""Tests for managers of contributions (media files)."""

import os
import glob

from PIL import Image
from io import BytesIO

from django.core.files.base import ContentFile
from django.test import TestCase
from django.conf import settings

from nose.tools import raises

from geokey.core.exceptions import FileTypeError
from geokey.core.tests.helpers.image_helpers import get_image
from geokey.contributions.models import MediaFile

from geokey.contributions.tests.model_factories import ObservationFactory
from geokey.users.tests.model_factories import UserFactory
from .model_factories import ImageFileFactory


class ModelManagerTest(TestCase):
    def tearDown(self):
        files = glob.glob(os.path.join(
            settings.MEDIA_ROOT,
            'user-uploads/images/*'
        ))
        for f in files:
            os.remove(f)

    def test_get_queryset(self):
        ImageFileFactory.create_batch(3)
        files = MediaFile.objects.all()

        self.assertEqual(len(files), 3)
        for f in files:
            self.assertEqual('ImageFile', f.type_name)

    def test_create_image(self):
        image_file = MediaFile.objects.create(
            name='Test name',
            description='Test Description',
            contribution=ObservationFactory.create(),
            creator=UserFactory.create(),
            the_file=get_image()
        )

        self.assertIsNotNone(image_file.image)
        self.assertEqual(image_file.type_name, 'ImageFile')

    @raises(FileTypeError)
    def test_create_not_supported(self):
        xyz_file = BytesIO()
        xyz = Image.new('RGBA', size=(50, 50), color=(256, 0, 0))
        xyz.save(xyz_file, 'png')
        xyz_file.seek(0)

        the_file = ContentFile(xyz_file.read(), 'test.xyz')
        the_file.content_type = 'chemical/x-xyz'

        MediaFile.objects.create(
            name='Test name',
            description='Test Description',
            contribution=ObservationFactory.create(),
            creator=UserFactory.create(),
            the_file=the_file
        )



# -*- coding: utf-8 -*-


from django.db import models, migrations
try:
    from django.contrib.postgres.fields import JSONField
except ImportError:
    from django_pgjson.fields import JsonBField as JSONField


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0002_auto_20150106_1420'),
    ]

    operations = [
        migrations.AddField(
            model_name='usergroup',
            name='filters',
            field=JSONField(null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='usergroup',
            name='where_clause',
            field=models.TextField(null=True, blank=True),
            preserve_default=True,
        ),
    ]



"""Model factories used for tests of socialinteractions."""

import factory

from allauth.socialaccount.models import SocialAccount

from geokey.users.tests.model_factories import UserFactory
from geokey.projects.tests.model_factories import ProjectFactory

from ..models import SocialInteractionPost, SocialInteractionPull


class SocialInteractionFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = SocialInteractionPost

    creator = factory.SubFactory(UserFactory)
    text_to_post = 'Text to post including $link$'
    link = 'www.link.com'
    project = factory.SubFactory(ProjectFactory)
    status = 'active'
    socialaccount = SocialAccount()


class SocialInteractionPullFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = SocialInteractionPull

    creator = factory.SubFactory(UserFactory)
    text_to_pull = '#Project2'
    project = factory.SubFactory(ProjectFactory)
    status = 'active'
    socialaccount = SocialAccount()
    frequency = '5min'



from django.conf import settings


def allowed_contributors(request):
    """Allows access to settings constants within templates."""
    _ = request  # PEP-8 compliance without reducing readability.
    default = ("true", "auth", "false")
    # return the value you want as a dictionary. you may add multiple values in there.
    if hasattr(settings, 'ALLOWED_CONTRIBUTORS'):
        return {'ALLOWED_CONTRIBUTORS': settings.ALLOWED_CONTRIBUTORS}
    else:
        return {'ALLOWED_CONTRIBUTORS': default}



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-02-24 15:49


from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contributions', '0017_auto_20160531_1434'),
    ]

    operations = [
        migrations.CreateModel(
            name='HistoricalComment',
            fields=[
                ('id', models.IntegerField(auto_created=True, blank=True, db_index=True, verbose_name='ID')),
                ('text', models.TextField()),
                ('created_at', models.DateTimeField(blank=True, editable=False)),
                ('status', models.CharField(choices=[(b'active', b'active'), (b'deleted', b'deleted')], default=b'active', max_length=20)),
                ('review_status', models.CharField(blank=True, choices=[(b'open', b'open'), (b'resolved', b'resolved')], max_length=10, null=True)),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_date', models.DateTimeField()),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('commentto', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='contributions.Observation')),
                ('creator', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('respondsto', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='contributions.Comment')),
            ],
            options={
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
                'verbose_name': 'historical comment',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-28 16:38
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('categories', '0018_historicalcategory'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='lookupvalue',
            options={'ordering': ['order']},
        ),
        migrations.AlterModelOptions(
            name='multiplelookupvalue',
            options={'ordering': ['order']},
        ),
        migrations.AddField(
            model_name='lookupvalue',
            name='order',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='multiplelookupvalue',
            name='order',
            field=models.IntegerField(default=0),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='email',
            field=models.EmailField(unique=True, max_length=254),
        ),
        migrations.AlterField(
            model_name='user',
            name='last_login',
            field=models.DateTimeField(null=True, verbose_name='last login', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('applications', '0002_auto_20150112_1807'),
    ]

    operations = [
        migrations.AlterField(
            model_name='application',
            name='user',
            field=models.ForeignKey(related_name='applications_application', blank=True, to=settings.AUTH_USER_MODEL, null=True),
        ),
    ]



"""utils."""
import os

from django.conf import settings
from django.apps import apps

import django

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "local_settings.settings")
django.setup()

import tweepy
import facebook

from allauth.socialaccount.models import SocialToken, SocialApp
from geokey.socialinteractions.models import SocialInteractionPull
from geokey.categories.models import Category, TextField, NumericField, Field
from geokey.contributions.models import Observation, Location

from datetime import timedelta
from geokey.socialinteractions.base import freq_dic

from django.utils import timezone


def check_dates(updated_at, frequency):
    """Check if data the SI Pull needs to be updated."""

    update = updated_at + timedelta(hours=1)
    now = timezone.now() + timedelta(hours=1)
    diff = (((now - update).total_seconds()) / 3600)

    if diff > freq_dic[frequency]:
        return True
    else:
        return False


def start2pull():
    """Start pulling data from Twitter."""
    si_pull_all = SocialInteractionPull.objects.filter(status='active')
    for si_pull in si_pull_all:
        socialaccount = si_pull.socialaccount
        provider = socialaccount.provider
        app = SocialApp.objects.get(provider=provider)
        access_token = SocialToken.objects.get(
            account__id=socialaccount.id,
            account__user=socialaccount.user,
            account__provider=app.provider
        )

        last_checked = si_pull.checked_at if si_pull.checked_at is not None else si_pull.created_at
        check_required = check_dates(last_checked, si_pull.frequency)

        if check_required:
            geo_tweets = pull_from_social_media(
                provider,
                access_token,
                si_pull.text_to_pull,
                si_pull.since_id,
                app)

            si_pull.checked_at = timezone.now()

            if len(geo_tweets) != 0:
                si_pull.since_id = max([x['id'] for x in geo_tweets])
                try:
                    project = si_pull.project
                except:
                    next

                tweet_category, text_field, tweet_id_field = get_category_and_field(
                    project,
                    socialaccount)
                for geo_tweet in geo_tweets:
                    if not Observation.objects.filter(project=project, status='active', properties__contains=geo_tweet['id']):
                        create_new_observation(
                            si_pull,
                            geo_tweet,
                            tweet_category,
                            text_field,
                            tweet_id_field
                        )
            si_pull.save()


def create_new_observation(si_pull, geo_tweet, tweet_category, text_field, tweet_id_field):
    """Create new observation based on the tweet.

    Parameters
    -----------
    si_pull: SocialInteractionPull

    geo_tweet: array of tweets

    tweet_category: Category Object

    text_field: TextField Object
    
    tweet_id_field: TweetID Object
    """
    point = geo_tweet['geometry']
    properties = {
        text_field.key: geo_tweet['text'],
        tweet_id_field.key: geo_tweet['id']
    }

    location = Location(
        geometry=point,
        creator=si_pull.socialaccount.user)

    Observation.create(
        properties=properties,
        location=location,
        project=si_pull.project,
        creator=si_pull.socialaccount.user,
        category=tweet_category,
        status='active')

    si_pull.updated_at = timezone.now()
    si_pull.save()


def get_category_and_field(project, socialaccount):
    """Check if Tweet category exists and text field exists.

    Parameters
    ------------
    project: Project Object

    socialaccount: socialaccount object

    Returns
    --------
    tweet_cat: Category object

    text_field: FieldText object

    """
    try:
        tweet_category = Category.objects.get(
            name="Tweets",
            project=project)

    except:
        tweet_category = Category.objects.create(
            name="Tweets",
            project=project,
            creator=socialaccount.user)

    if TextField.objects.filter(category=tweet_category, key='tweet'):

        text_field = TextField.objects.get(
            category=tweet_category,
            key='tweet')
    else:
        text_field = TextField.objects.create(
            name='Tweet',
            category=tweet_category,
            key="tweet")

    if NumericField.objects.filter(category=tweet_category, key='tweet-id'):

        tweet_id_field = NumericField.objects.get(
            category=tweet_category,
            key='tweet-id')
    else:
        tweet_id_field = NumericField.objects.create(
            name='Tweet-ID',
            category=tweet_category,
            key='tweet-id'
        )
        field = Field.objects.get(pk=tweet_id_field.field_ptr_id)
        field.order = 1
        field.save()
    tweet_category.display_field = text_field
    tweet_category.save()

    return tweet_category, text_field, tweet_id_field


def pull_from_social_media(provider, access_token, text_to_pull, tweet_id, app):
    """Pull data from the timeline with specific text.

    Parameters
    -----------
    provider =  str
        provider of the social account

    access_token: str - SocialToken Object
        access token for the social account and user
    text_to_pull: str
        text to be searched on the tweets
    tweet_id: int
        searches all tweets with id more recent than tweet_id
    app: socialAccount app object
    Returns
    --------
    geo_tweets: array
        array of geo-referenced tweet objects
    """
    if provider == 'twitter':
        try:
            consumer_key = app.client_id
            consumer_secret = app.secret
            auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
            access_token_all = access_token
            access_token = access_token_all.token
            access_token_secret = access_token_all.token_secret
            auth.set_access_token(access_token, access_token_secret)
            api = tweepy.API(auth)

            try:
                tweets_all = api.home_timeline(count=100, since_id=tweet_id, tweet_mode='extended')
            except Exception:
                return "Impossible to get data from the timeline"
        except:
            return "You are not authenticated"

        geo_tweets = []
        for tweet in tweets_all:
            if text_to_pull.lower() in tweet.full_text.lower():
                if tweet.coordinates or tweet.place:
                    new_contribution = {}
                    new_contribution['id'] = tweet.id
                    new_contribution['text'] = tweet.full_text
                    new_contribution['user'] = tweet.user.name
                    new_contribution['created_at'] = tweet.created_at
                    new_contribution['geometry'] = process_location(tweet)
                    geo_tweets.append(new_contribution)
        geo_tweets.reverse()
    return geo_tweets


def process_location(tweet):
    """Retrieve coordinates or place coordinates from tweet and converts them into WKT Point or Polygon.

    Parameters
    -----------
    tweet =  JSON
        tweet object returned by API

    Returns
    --------
    wkt_coordinates: str
        coordinates in WKT format
    """
    wkt_coordinates = ""
    if tweet.coordinates:
        coordinates = tweet.coordinates
        wkt_coordinates = 'POINT(' + str(coordinates['coordinates'][0]) + ' ' + str(coordinates['coordinates'][1]) + ')'
    elif tweet.place:
        coordinates = tweet.place.bounding_box.coordinates

        is_point = all(c == coordinates[0] for c in coordinates)
        if is_point:
            wkt_coordinates = 'POINT(' + str(coordinates[0][0][0]) + ' ' + str(coordinates[0][0][1]) + ')'
        else:
            wkt_coordinates = 'POLYGON(' + \
                              str(coordinates[0][0][0]) + ' ' + str(coordinates[0][0][1]) + + ', ' + \
                              str(coordinates[0][1][0]) + ' ' + str(coordinates[0][1][1]) + + ', ' + \
                              str(coordinates[0][2][0]) + ' ' + str(coordinates[0][2][1]) + + ', ' + \
                              str(coordinates[0][3][0]) + ' ' + str(coordinates[0][3][1]) + + ', ' + ')'

    return wkt_coordinates



"""Test all utils."""

from django.test import TestCase

from oauth2client.client import OAuth2WebServerFlow

from geokey.contributions.utils import (
    my_flow_from_clientsecrets,
    get_args,
    get_authenticated_service,
    initialize_upload
)


class GetArgsTest(TestCase):
    """Test for method 'get_args'."""

    def setUp(self):
        """Set up tests."""
        self.path = 'path/sample/test/test_file.mp4'
        self.name = 'test_file'

    def test_method(self):
        """Test method."""
        args = get_args(self.name, self.path)
        args_var = vars(args)

        self.assertEqual(args_var['file'], self.path)

        self.assertEqual(args_var['title'], self.name)


class MyFlowFromClientSecretTest(TestCase):
    """Test for method 'my_flow_from_clientsecrets'."""

    def setUp(self):
        """Set up tests."""
        self.youtube_uploader = {
            'scope': "https://www.googleapis.com/auth/youtube.upload",
            'auth_host_name': 'localhost',
            'auth_host_port': [8080, 8000],
            'client_info': {
                "client_id": "109430273076-t3e30ie5aseb3laj2da0gkpikir6b0e9.apps.googleusercontent.com",
                "client_secret": "o3U69gnO4FRipA1Q3K6gi0_N",
                "redirect_uris": ["http://localhost"],
                "auth_uri": "https://accounts.google.com/o/oauth2/auth",
                "token_uri": "https://accounts.google.com/o/oauth2/token"
            },

        }

    def test_method(self):
        """Test method."""
        flow = my_flow_from_clientsecrets(
            self.youtube_uploader['client_info'],
            self.youtube_uploader['scope'])

        client_info = self.youtube_uploader['client_info']

        constructor_kwargs = {
            'redirect_uri': None,
            'auth_uri': client_info['auth_uri'],
            'token_uri': client_info['token_uri'],
            'login_hint': None,
        }
        flow_new = OAuth2WebServerFlow(
            client_info['client_id'], client_info['client_secret'],
            self.youtube_uploader['scope'], **constructor_kwargs)
        self.assertEqual(flow.client_id, flow_new.client_id)
        self.assertEqual(flow.scope, flow_new.scope)
        self.assertEqual(flow.client_secret, flow_new.client_secret)



"""Custom GeoKey template tags."""

from django import template

register = template.Library()


@register.filter()
def check_media_file_type(media_file_class):
    """Check media file type and returns in a human-readble format."""
    if media_file_class == 'AudioFile':
        media_file_type = 'Audio file'
    elif media_file_class == 'VideoFile':
        media_file_type = 'Video file'
    elif media_file_class == 'DocumentFile':
        media_file_type = 'Document file'
    elif media_file_class == 'ImageFile':
        media_file_type = 'Image file'

    return media_file_type


@register.filter()
def check_field_type(field_class):
    """Check field type and returns in a human-readble format."""
    if field_class == 'TextField':
        field_type = 'Text field'
    elif field_class == 'NumericField':
        field_type = 'Numeric field'
    elif field_class == 'DateField':
        field_type = 'Date field'
    elif field_class == 'DateTimeField':
        field_type = 'Date & time field'
    elif field_class == 'TimeField':
        field_type = 'Time field'
    elif field_class == 'LookupField':
        field_type = 'Select box field'
    elif field_class == 'MultipleLookupField':
        field_type = 'Multiple select field'

    return field_type



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('socialaccount', '0003_extra_data_default_dict'),
        ('projects', '0008_historicalproject'),
    ]

    operations = [
        migrations.CreateModel(
            name='SocialInteractionPost',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('text_to_post', models.TextField(null=True, blank=True)),
                ('link', models.TextField(null=True, blank=True)),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'inactive', b'inactive')])),
                ('creator', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
                ('project', models.ForeignKey(related_name='socialinteractions_post', to='projects.Project')),
                ('socialaccount', models.ForeignKey(related_name='socialinteractions_post', to='socialaccount.SocialAccount')),
            ],
        ),
        migrations.CreateModel(
            name='SocialInteractionPull',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('text_to_pull', models.TextField(null=True, blank=True)),
                ('frequency', models.CharField(default=b'daily', max_length=20, choices=[(b'5min', b'5min'), (b'10min', b'10min'), (b'20min', b'20min'), (b'30min', b'30min'), (b'hourly', b'hourly'), (b'daily', b'daily'), (b'weekly', b'weekly'), (b'fortnightly', b'fortnightly'), (b'monthly', b'monthly')])),
                ('status', models.CharField(default=b'active', max_length=20, choices=[(b'active', b'active'), (b'inactive', b'inactive')])),
                ('since_id', models.TextField(null=True, blank=True)),
                ('updated_at', models.DateTimeField(null=True)),
                ('checked_at', models.DateTimeField(null=True)),
                ('creator', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
                ('project', models.ForeignKey(related_name='socialinteractions_pull', to='projects.Project')),
                ('socialaccount', models.ForeignKey(related_name='socialinteractions_pull', to='socialaccount.SocialAccount')),
            ],
        ),
    ]



"""Core views."""

from rest_framework.views import APIView
from rest_framework.response import Response

from django.views.generic import TemplateView

from braces.views import LoginRequiredMixin

from geokey.version import get_version
from geokey.extensions.base import extensions
from geokey.projects.views import ProjectContext
from geokey.core.models import LoggerHistory

from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger


class LoggerList(LoginRequiredMixin, ProjectContext, TemplateView):
    """A list of all history logs."""

    template_name = 'logger/logger_list.html'

    def get_context_data(self, project_id, *args, **kwargs):
        """
        Return the context to render the view.
        Parameters
        ----------
        project_id : int
            Identifies the project in the database.
        Returns
        -------
        dict
            Context.
        """

        context = super(LoggerList, self).get_context_data(
            project_id,
            *args,
            **kwargs
        )

        logs = LoggerHistory.objects.filter(
            project__contains={'id': str(project_id)})

        context['logs'] = self.paginate_logs(
            logs[::-1],
            self.request.GET.get('page'))

        return context

    def paginate_logs(self, logs, page):
            """Paginate all logs."""
            paginator = Paginator(logs, 20)

            try:
                logs = paginator.page(page)
            except PageNotAnInteger:
                logs = paginator.page(1)
            except EmptyPage:
                logs = paginator.page(paginator.num_pages)

            return logs


# ############################################################################
#
# PUBLIC API
#
# ############################################################################

class InfoAPIView(APIView):
    """Public API for GeoKey server information."""

    def get(self, request):
        """
        Handle GET request.

        Return GeoKey server information.

        Parameters
        ----------
        request : rest_framework.request.Request
            Object representing the request.

        Returns
        -------
        rest_framework.response.Response
            Contains the GeoKey server information.
        """
        info = {'geokey': {}}
        info['geokey']['version'] = get_version()
        info['geokey']['installed_extensions'] = [{
                'name': ext_id_ext[0],
                'version': ext_id_ext[1]['version'] if 'version' in ext_id_ext[1] else None
            } for ext_id_ext in iter(list(extensions.items()))]

        return Response(info)
